# Example Praefect configuration file

# # TCP address to listen on
listen_addr = "127.0.0.1:2305"

# # Praefect can listen on a socket when placed on the same machine as all clients
# socket_path = "/home/git/gitlab/tmp/sockets/private/praefect.socket"
# # Optional: export metrics via Prometheus
# prometheus_listen_addr = "127.0.01:10101"
# # You can optionally configure Praefect to output JSON-formatted log messages to stdout
# [logging]
#   format = "json"
# # Optional: Set log level to only log entries with that severity or above
# # One of, in order: debug, info, warn, errror, fatal, panic
# # Defaults to "info"
#   level = "warn"
# [sentry]
#   sentry_environment = ""
#   sentry_dsn = ""
#
#  Optional: authenticate Gitaly requests using a shared secret. This token works the same way as a gitaly token
# [auth]
#   token = 'abc123secret'
#
# # One or more Gitaly servers need to be configured to be managed. The names
# of each server are used to link multiple nodes, or `gitaly_server`s together
# as shard. listen_addr should be unique for all nodes.
# Requires the protocol to be defined, e.g. tcp://host.tld:1234

[[virtual_storage]]
name = "praefect"

[[virtual_storage.node]]
  storage = "praefect-git-0"
  address = "tcp://praefect-git-0.internal"
  token = 'token1'

[[virtual_storage.node]]
  storage = "praefect-git-1"
  address = "tcp://praefect-git-1.internal"
  token = 'token2'

[[virtual_storage.node]]
  storage = "praefect-git-2"
  address = "tcp://praefect-git-2.internal"
  token = 'token3'

[database]
# In CI this magical hostname 'postgres' points to our dedicated CI postgres instance.
host = 'postgres'

# POSTGRES_* variables are defined in gitlab-ci.yml
dbname = '<%= ENV['POSTGRES_DB'] %>'
user = '<%= ENV['POSTGRES_USER'] %>'
password = '<%= ENV['POSTGRES_PASSWORD'] %>'

# No SSL in CI
sslmode = 'disable'
