// Code generated by protoc-gen-go. DO NOT EDIT.
// source: server.proto

package gitalypb

import (
	context "context"
	fmt "fmt"
	proto "github.com/golang/protobuf/proto"
	grpc "google.golang.org/grpc"
	codes "google.golang.org/grpc/codes"
	status "google.golang.org/grpc/status"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

type ServerInfoRequest struct {
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *ServerInfoRequest) Reset()         { *m = ServerInfoRequest{} }
func (m *ServerInfoRequest) String() string { return proto.CompactTextString(m) }
func (*ServerInfoRequest) ProtoMessage()    {}
func (*ServerInfoRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_ad098daeda4239f7, []int{0}
}

func (m *ServerInfoRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ServerInfoRequest.Unmarshal(m, b)
}
func (m *ServerInfoRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ServerInfoRequest.Marshal(b, m, deterministic)
}
func (m *ServerInfoRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ServerInfoRequest.Merge(m, src)
}
func (m *ServerInfoRequest) XXX_Size() int {
	return xxx_messageInfo_ServerInfoRequest.Size(m)
}
func (m *ServerInfoRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_ServerInfoRequest.DiscardUnknown(m)
}

var xxx_messageInfo_ServerInfoRequest proto.InternalMessageInfo

type ServerInfoResponse struct {
	ServerVersion        string                              `protobuf:"bytes,1,opt,name=server_version,json=serverVersion,proto3" json:"server_version,omitempty"`
	GitVersion           string                              `protobuf:"bytes,2,opt,name=git_version,json=gitVersion,proto3" json:"git_version,omitempty"`
	StorageStatuses      []*ServerInfoResponse_StorageStatus `protobuf:"bytes,3,rep,name=storage_statuses,json=storageStatuses,proto3" json:"storage_statuses,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                            `json:"-"`
	XXX_unrecognized     []byte                              `json:"-"`
	XXX_sizecache        int32                               `json:"-"`
}

func (m *ServerInfoResponse) Reset()         { *m = ServerInfoResponse{} }
func (m *ServerInfoResponse) String() string { return proto.CompactTextString(m) }
func (*ServerInfoResponse) ProtoMessage()    {}
func (*ServerInfoResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_ad098daeda4239f7, []int{1}
}

func (m *ServerInfoResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ServerInfoResponse.Unmarshal(m, b)
}
func (m *ServerInfoResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ServerInfoResponse.Marshal(b, m, deterministic)
}
func (m *ServerInfoResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ServerInfoResponse.Merge(m, src)
}
func (m *ServerInfoResponse) XXX_Size() int {
	return xxx_messageInfo_ServerInfoResponse.Size(m)
}
func (m *ServerInfoResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_ServerInfoResponse.DiscardUnknown(m)
}

var xxx_messageInfo_ServerInfoResponse proto.InternalMessageInfo

func (m *ServerInfoResponse) GetServerVersion() string {
	if m != nil {
		return m.ServerVersion
	}
	return ""
}

func (m *ServerInfoResponse) GetGitVersion() string {
	if m != nil {
		return m.GitVersion
	}
	return ""
}

func (m *ServerInfoResponse) GetStorageStatuses() []*ServerInfoResponse_StorageStatus {
	if m != nil {
		return m.StorageStatuses
	}
	return nil
}

type ServerInfoResponse_StorageStatus struct {
	StorageName          string   `protobuf:"bytes,1,opt,name=storage_name,json=storageName,proto3" json:"storage_name,omitempty"`
	Readable             bool     `protobuf:"varint,2,opt,name=readable,proto3" json:"readable,omitempty"`
	Writeable            bool     `protobuf:"varint,3,opt,name=writeable,proto3" json:"writeable,omitempty"`
	FsType               string   `protobuf:"bytes,4,opt,name=fs_type,json=fsType,proto3" json:"fs_type,omitempty"`
	FilesystemId         string   `protobuf:"bytes,5,opt,name=filesystem_id,json=filesystemId,proto3" json:"filesystem_id,omitempty"`
	ReplicationFactor    uint32   `protobuf:"varint,6,opt,name=replication_factor,json=replicationFactor,proto3" json:"replication_factor,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *ServerInfoResponse_StorageStatus) Reset()         { *m = ServerInfoResponse_StorageStatus{} }
func (m *ServerInfoResponse_StorageStatus) String() string { return proto.CompactTextString(m) }
func (*ServerInfoResponse_StorageStatus) ProtoMessage()    {}
func (*ServerInfoResponse_StorageStatus) Descriptor() ([]byte, []int) {
	return fileDescriptor_ad098daeda4239f7, []int{1, 0}
}

func (m *ServerInfoResponse_StorageStatus) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ServerInfoResponse_StorageStatus.Unmarshal(m, b)
}
func (m *ServerInfoResponse_StorageStatus) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ServerInfoResponse_StorageStatus.Marshal(b, m, deterministic)
}
func (m *ServerInfoResponse_StorageStatus) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ServerInfoResponse_StorageStatus.Merge(m, src)
}
func (m *ServerInfoResponse_StorageStatus) XXX_Size() int {
	return xxx_messageInfo_ServerInfoResponse_StorageStatus.Size(m)
}
func (m *ServerInfoResponse_StorageStatus) XXX_DiscardUnknown() {
	xxx_messageInfo_ServerInfoResponse_StorageStatus.DiscardUnknown(m)
}

var xxx_messageInfo_ServerInfoResponse_StorageStatus proto.InternalMessageInfo

func (m *ServerInfoResponse_StorageStatus) GetStorageName() string {
	if m != nil {
		return m.StorageName
	}
	return ""
}

func (m *ServerInfoResponse_StorageStatus) GetReadable() bool {
	if m != nil {
		return m.Readable
	}
	return false
}

func (m *ServerInfoResponse_StorageStatus) GetWriteable() bool {
	if m != nil {
		return m.Writeable
	}
	return false
}

func (m *ServerInfoResponse_StorageStatus) GetFsType() string {
	if m != nil {
		return m.FsType
	}
	return ""
}

func (m *ServerInfoResponse_StorageStatus) GetFilesystemId() string {
	if m != nil {
		return m.FilesystemId
	}
	return ""
}

func (m *ServerInfoResponse_StorageStatus) GetReplicationFactor() uint32 {
	if m != nil {
		return m.ReplicationFactor
	}
	return 0
}

type DiskStatisticsRequest struct {
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *DiskStatisticsRequest) Reset()         { *m = DiskStatisticsRequest{} }
func (m *DiskStatisticsRequest) String() string { return proto.CompactTextString(m) }
func (*DiskStatisticsRequest) ProtoMessage()    {}
func (*DiskStatisticsRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_ad098daeda4239f7, []int{2}
}

func (m *DiskStatisticsRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_DiskStatisticsRequest.Unmarshal(m, b)
}
func (m *DiskStatisticsRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_DiskStatisticsRequest.Marshal(b, m, deterministic)
}
func (m *DiskStatisticsRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_DiskStatisticsRequest.Merge(m, src)
}
func (m *DiskStatisticsRequest) XXX_Size() int {
	return xxx_messageInfo_DiskStatisticsRequest.Size(m)
}
func (m *DiskStatisticsRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_DiskStatisticsRequest.DiscardUnknown(m)
}

var xxx_messageInfo_DiskStatisticsRequest proto.InternalMessageInfo

type DiskStatisticsResponse struct {
	StorageStatuses      []*DiskStatisticsResponse_StorageStatus `protobuf:"bytes,1,rep,name=storage_statuses,json=storageStatuses,proto3" json:"storage_statuses,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                                `json:"-"`
	XXX_unrecognized     []byte                                  `json:"-"`
	XXX_sizecache        int32                                   `json:"-"`
}

func (m *DiskStatisticsResponse) Reset()         { *m = DiskStatisticsResponse{} }
func (m *DiskStatisticsResponse) String() string { return proto.CompactTextString(m) }
func (*DiskStatisticsResponse) ProtoMessage()    {}
func (*DiskStatisticsResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_ad098daeda4239f7, []int{3}
}

func (m *DiskStatisticsResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_DiskStatisticsResponse.Unmarshal(m, b)
}
func (m *DiskStatisticsResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_DiskStatisticsResponse.Marshal(b, m, deterministic)
}
func (m *DiskStatisticsResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_DiskStatisticsResponse.Merge(m, src)
}
func (m *DiskStatisticsResponse) XXX_Size() int {
	return xxx_messageInfo_DiskStatisticsResponse.Size(m)
}
func (m *DiskStatisticsResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_DiskStatisticsResponse.DiscardUnknown(m)
}

var xxx_messageInfo_DiskStatisticsResponse proto.InternalMessageInfo

func (m *DiskStatisticsResponse) GetStorageStatuses() []*DiskStatisticsResponse_StorageStatus {
	if m != nil {
		return m.StorageStatuses
	}
	return nil
}

type DiskStatisticsResponse_StorageStatus struct {
	// When both available and used fields are equal 0 that means that
	// Gitaly was unable to determine storage stats.
	StorageName          string   `protobuf:"bytes,1,opt,name=storage_name,json=storageName,proto3" json:"storage_name,omitempty"`
	Available            int64    `protobuf:"varint,2,opt,name=available,proto3" json:"available,omitempty"`
	Used                 int64    `protobuf:"varint,3,opt,name=used,proto3" json:"used,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *DiskStatisticsResponse_StorageStatus) Reset()         { *m = DiskStatisticsResponse_StorageStatus{} }
func (m *DiskStatisticsResponse_StorageStatus) String() string { return proto.CompactTextString(m) }
func (*DiskStatisticsResponse_StorageStatus) ProtoMessage()    {}
func (*DiskStatisticsResponse_StorageStatus) Descriptor() ([]byte, []int) {
	return fileDescriptor_ad098daeda4239f7, []int{3, 0}
}

func (m *DiskStatisticsResponse_StorageStatus) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_DiskStatisticsResponse_StorageStatus.Unmarshal(m, b)
}
func (m *DiskStatisticsResponse_StorageStatus) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_DiskStatisticsResponse_StorageStatus.Marshal(b, m, deterministic)
}
func (m *DiskStatisticsResponse_StorageStatus) XXX_Merge(src proto.Message) {
	xxx_messageInfo_DiskStatisticsResponse_StorageStatus.Merge(m, src)
}
func (m *DiskStatisticsResponse_StorageStatus) XXX_Size() int {
	return xxx_messageInfo_DiskStatisticsResponse_StorageStatus.Size(m)
}
func (m *DiskStatisticsResponse_StorageStatus) XXX_DiscardUnknown() {
	xxx_messageInfo_DiskStatisticsResponse_StorageStatus.DiscardUnknown(m)
}

var xxx_messageInfo_DiskStatisticsResponse_StorageStatus proto.InternalMessageInfo

func (m *DiskStatisticsResponse_StorageStatus) GetStorageName() string {
	if m != nil {
		return m.StorageName
	}
	return ""
}

func (m *DiskStatisticsResponse_StorageStatus) GetAvailable() int64 {
	if m != nil {
		return m.Available
	}
	return 0
}

func (m *DiskStatisticsResponse_StorageStatus) GetUsed() int64 {
	if m != nil {
		return m.Used
	}
	return 0
}

func init() {
	proto.RegisterType((*ServerInfoRequest)(nil), "gitaly.ServerInfoRequest")
	proto.RegisterType((*ServerInfoResponse)(nil), "gitaly.ServerInfoResponse")
	proto.RegisterType((*ServerInfoResponse_StorageStatus)(nil), "gitaly.ServerInfoResponse.StorageStatus")
	proto.RegisterType((*DiskStatisticsRequest)(nil), "gitaly.DiskStatisticsRequest")
	proto.RegisterType((*DiskStatisticsResponse)(nil), "gitaly.DiskStatisticsResponse")
	proto.RegisterType((*DiskStatisticsResponse_StorageStatus)(nil), "gitaly.DiskStatisticsResponse.StorageStatus")
}

func init() { proto.RegisterFile("server.proto", fileDescriptor_ad098daeda4239f7) }

var fileDescriptor_ad098daeda4239f7 = []byte{
	// 460 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0x9c, 0x93, 0x5f, 0x6e, 0xd3, 0x40,
	0x10, 0xc6, 0x65, 0x1c, 0x42, 0x33, 0x49, 0x0a, 0x1d, 0x04, 0x35, 0x56, 0x81, 0x10, 0x84, 0xe4,
	0x07, 0x6a, 0x8b, 0xd0, 0x13, 0x00, 0x42, 0xea, 0x0b, 0x48, 0x0e, 0x02, 0x89, 0x17, 0x6b, 0x63,
	0x8f, 0xad, 0x15, 0x8e, 0xd7, 0xec, 0x6c, 0x82, 0x72, 0x08, 0x9e, 0x39, 0x04, 0x07, 0x42, 0xdc,
	0x82, 0x23, 0xa0, 0xac, 0xf3, 0xaf, 0x90, 0x82, 0xd4, 0x17, 0x6b, 0xf7, 0xfb, 0x66, 0x76, 0x67,
	0x7f, 0x33, 0x86, 0x1e, 0x93, 0x9e, 0x93, 0x0e, 0x6b, 0xad, 0x8c, 0xc2, 0x76, 0x21, 0x8d, 0x28,
	0x17, 0x3e, 0x94, 0xb2, 0x32, 0x8d, 0x36, 0xbc, 0x0d, 0x47, 0x63, 0x1b, 0x73, 0x5e, 0xe5, 0x2a,
	0xa6, 0xcf, 0x33, 0x62, 0x33, 0xfc, 0xea, 0x02, 0xee, 0xaa, 0x5c, 0xab, 0x8a, 0x09, 0x9f, 0xc0,
	0x61, 0x73, 0x5e, 0x32, 0x27, 0xcd, 0x52, 0x55, 0x9e, 0x33, 0x70, 0x82, 0x4e, 0xdc, 0x6f, 0xd4,
	0xf7, 0x8d, 0x88, 0x0f, 0xa1, 0x5b, 0x48, 0xb3, 0x89, 0xb9, 0x66, 0x63, 0xa0, 0x90, 0x66, 0x1d,
	0x30, 0x86, 0x5b, 0x6c, 0x94, 0x16, 0x05, 0x25, 0x6c, 0x84, 0x99, 0x31, 0xb1, 0xe7, 0x0e, 0xdc,
	0xa0, 0x3b, 0x0a, 0xc2, 0xa6, 0xc4, 0xf0, 0xef, 0xdb, 0xc3, 0x71, 0x93, 0x32, 0xb6, 0x19, 0xf1,
	0x4d, 0xde, 0xdd, 0x12, 0xfb, 0x3f, 0x1d, 0xe8, 0x5f, 0x08, 0xc1, 0x47, 0xd0, 0x5b, 0x5f, 0x53,
	0x89, 0x29, 0xad, 0x8a, 0xed, 0xae, 0xb4, 0x37, 0x62, 0x4a, 0xe8, 0xc3, 0x81, 0x26, 0x91, 0x89,
	0x49, 0x49, 0xb6, 0xce, 0x83, 0x78, 0xb3, 0xc7, 0x13, 0xe8, 0x7c, 0xd1, 0xd2, 0x90, 0x35, 0x5d,
	0x6b, 0x6e, 0x05, 0x3c, 0x86, 0x1b, 0x39, 0x27, 0x66, 0x51, 0x93, 0xd7, 0xb2, 0xe7, 0xb6, 0x73,
	0x7e, 0xb7, 0xa8, 0x09, 0x1f, 0x43, 0x3f, 0x97, 0x25, 0xf1, 0x82, 0x0d, 0x4d, 0x13, 0x99, 0x79,
	0xd7, 0xad, 0xdd, 0xdb, 0x8a, 0xe7, 0x19, 0x9e, 0x02, 0x6a, 0xaa, 0x4b, 0x99, 0x0a, 0x23, 0x55,
	0x95, 0xe4, 0x22, 0x35, 0x4a, 0x7b, 0xed, 0x81, 0x13, 0xf4, 0xe3, 0xa3, 0x1d, 0xe7, 0xb5, 0x35,
	0x86, 0xc7, 0x70, 0xe7, 0x95, 0xe4, 0x4f, 0xcb, 0x77, 0x49, 0x36, 0x32, 0xe5, 0x75, 0xa3, 0x7e,
	0x38, 0x70, 0xf7, 0x4f, 0x67, 0xd5, 0xac, 0x0f, 0x7b, 0x20, 0x3b, 0x16, 0xf2, 0xd3, 0x35, 0xe4,
	0xfd, 0x99, 0xff, 0x03, 0x9d, 0x5d, 0x81, 0xf3, 0x09, 0x74, 0xc4, 0x5c, 0xc8, 0x72, 0x03, 0xda,
	0x8d, 0xb7, 0x02, 0x22, 0xb4, 0x66, 0x4c, 0x99, 0x85, 0xec, 0xc6, 0x76, 0x3d, 0xfa, 0xbe, 0x6c,
	0xa7, 0x1d, 0x82, 0xe5, 0x57, 0xa6, 0x84, 0x2f, 0x01, 0xb6, 0x53, 0x81, 0xf7, 0xf6, 0x4d, 0x8a,
	0x85, 0xe2, 0xfb, 0x97, 0x0f, 0x11, 0xbe, 0x85, 0xc3, 0x8b, 0xaf, 0xc6, 0xfb, 0x97, 0xd1, 0x68,
	0x0e, 0x7b, 0xf0, 0x6f, 0x58, 0x7e, 0xeb, 0xd7, 0xb7, 0xc0, 0x79, 0x71, 0xf6, 0x71, 0x54, 0x48,
	0x53, 0x8a, 0x49, 0x98, 0xaa, 0x69, 0xd4, 0x2c, 0x4f, 0x95, 0x2e, 0xa2, 0x26, 0x39, 0x9a, 0x3f,
	0x3b, 0x8b, 0xec, 0xff, 0x16, 0x15, 0x6a, 0xa5, 0xd5, 0x93, 0x49, 0xdb, 0x4a, 0xcf, 0x7f, 0x07,
	0x00, 0x00, 0xff, 0xff, 0x62, 0x79, 0xb4, 0x17, 0xa6, 0x03, 0x00, 0x00,
}

// Reference imports to suppress errors if they are not otherwise used.
var _ context.Context
var _ grpc.ClientConn

// This is a compile-time assertion to ensure that this generated file
// is compatible with the grpc package it is being compiled against.
const _ = grpc.SupportPackageIsVersion4

// ServerServiceClient is the client API for ServerService service.
//
// For semantics around ctx use and closing/ending streaming RPCs, please refer to https://godoc.org/google.golang.org/grpc#ClientConn.NewStream.
type ServerServiceClient interface {
	ServerInfo(ctx context.Context, in *ServerInfoRequest, opts ...grpc.CallOption) (*ServerInfoResponse, error)
	DiskStatistics(ctx context.Context, in *DiskStatisticsRequest, opts ...grpc.CallOption) (*DiskStatisticsResponse, error)
}

type serverServiceClient struct {
	cc *grpc.ClientConn
}

func NewServerServiceClient(cc *grpc.ClientConn) ServerServiceClient {
	return &serverServiceClient{cc}
}

func (c *serverServiceClient) ServerInfo(ctx context.Context, in *ServerInfoRequest, opts ...grpc.CallOption) (*ServerInfoResponse, error) {
	out := new(ServerInfoResponse)
	err := c.cc.Invoke(ctx, "/gitaly.ServerService/ServerInfo", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *serverServiceClient) DiskStatistics(ctx context.Context, in *DiskStatisticsRequest, opts ...grpc.CallOption) (*DiskStatisticsResponse, error) {
	out := new(DiskStatisticsResponse)
	err := c.cc.Invoke(ctx, "/gitaly.ServerService/DiskStatistics", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

// ServerServiceServer is the server API for ServerService service.
type ServerServiceServer interface {
	ServerInfo(context.Context, *ServerInfoRequest) (*ServerInfoResponse, error)
	DiskStatistics(context.Context, *DiskStatisticsRequest) (*DiskStatisticsResponse, error)
}

// UnimplementedServerServiceServer can be embedded to have forward compatible implementations.
type UnimplementedServerServiceServer struct {
}

func (*UnimplementedServerServiceServer) ServerInfo(ctx context.Context, req *ServerInfoRequest) (*ServerInfoResponse, error) {
	return nil, status.Errorf(codes.Unimplemented, "method ServerInfo not implemented")
}
func (*UnimplementedServerServiceServer) DiskStatistics(ctx context.Context, req *DiskStatisticsRequest) (*DiskStatisticsResponse, error) {
	return nil, status.Errorf(codes.Unimplemented, "method DiskStatistics not implemented")
}

func RegisterServerServiceServer(s *grpc.Server, srv ServerServiceServer) {
	s.RegisterService(&_ServerService_serviceDesc, srv)
}

func _ServerService_ServerInfo_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(ServerInfoRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ServerServiceServer).ServerInfo(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/gitaly.ServerService/ServerInfo",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ServerServiceServer).ServerInfo(ctx, req.(*ServerInfoRequest))
	}
	return interceptor(ctx, in, info, handler)
}

func _ServerService_DiskStatistics_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(DiskStatisticsRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ServerServiceServer).DiskStatistics(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/gitaly.ServerService/DiskStatistics",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ServerServiceServer).DiskStatistics(ctx, req.(*DiskStatisticsRequest))
	}
	return interceptor(ctx, in, info, handler)
}

var _ServerService_serviceDesc = grpc.ServiceDesc{
	ServiceName: "gitaly.ServerService",
	HandlerType: (*ServerServiceServer)(nil),
	Methods: []grpc.MethodDesc{
		{
			MethodName: "ServerInfo",
			Handler:    _ServerService_ServerInfo_Handler,
		},
		{
			MethodName: "DiskStatistics",
			Handler:    _ServerService_DiskStatistics_Handler,
		},
	},
	Streams:  []grpc.StreamDesc{},
	Metadata: "server.proto",
}
