// Code generated by protoc-gen-go. DO NOT EDIT.
// source: wiki.proto

package gitalypb

import (
	context "context"
	fmt "fmt"
	proto "github.com/golang/protobuf/proto"
	grpc "google.golang.org/grpc"
	codes "google.golang.org/grpc/codes"
	status "google.golang.org/grpc/status"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

type WikiGetAllPagesRequest_SortBy int32

const (
	WikiGetAllPagesRequest_TITLE      WikiGetAllPagesRequest_SortBy = 0
	WikiGetAllPagesRequest_CREATED_AT WikiGetAllPagesRequest_SortBy = 1
)

var WikiGetAllPagesRequest_SortBy_name = map[int32]string{
	0: "TITLE",
	1: "CREATED_AT",
}

var WikiGetAllPagesRequest_SortBy_value = map[string]int32{
	"TITLE":      0,
	"CREATED_AT": 1,
}

func (x WikiGetAllPagesRequest_SortBy) String() string {
	return proto.EnumName(WikiGetAllPagesRequest_SortBy_name, int32(x))
}

func (WikiGetAllPagesRequest_SortBy) EnumDescriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{9, 0}
}

type WikiListPagesRequest_SortBy int32

const (
	WikiListPagesRequest_TITLE      WikiListPagesRequest_SortBy = 0
	WikiListPagesRequest_CREATED_AT WikiListPagesRequest_SortBy = 1
)

var WikiListPagesRequest_SortBy_name = map[int32]string{
	0: "TITLE",
	1: "CREATED_AT",
}

var WikiListPagesRequest_SortBy_value = map[string]int32{
	"TITLE":      0,
	"CREATED_AT": 1,
}

func (x WikiListPagesRequest_SortBy) String() string {
	return proto.EnumName(WikiListPagesRequest_SortBy_name, int32(x))
}

func (WikiListPagesRequest_SortBy) EnumDescriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{11, 0}
}

type WikiCommitDetails struct {
	Name                 []byte   `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Email                []byte   `protobuf:"bytes,2,opt,name=email,proto3" json:"email,omitempty"`
	Message              []byte   `protobuf:"bytes,3,opt,name=message,proto3" json:"message,omitempty"`
	UserId               int32    `protobuf:"varint,4,opt,name=user_id,json=userId,proto3" json:"user_id,omitempty"`
	UserName             []byte   `protobuf:"bytes,5,opt,name=user_name,json=userName,proto3" json:"user_name,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *WikiCommitDetails) Reset()         { *m = WikiCommitDetails{} }
func (m *WikiCommitDetails) String() string { return proto.CompactTextString(m) }
func (*WikiCommitDetails) ProtoMessage()    {}
func (*WikiCommitDetails) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{0}
}

func (m *WikiCommitDetails) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiCommitDetails.Unmarshal(m, b)
}
func (m *WikiCommitDetails) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiCommitDetails.Marshal(b, m, deterministic)
}
func (m *WikiCommitDetails) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiCommitDetails.Merge(m, src)
}
func (m *WikiCommitDetails) XXX_Size() int {
	return xxx_messageInfo_WikiCommitDetails.Size(m)
}
func (m *WikiCommitDetails) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiCommitDetails.DiscardUnknown(m)
}

var xxx_messageInfo_WikiCommitDetails proto.InternalMessageInfo

func (m *WikiCommitDetails) GetName() []byte {
	if m != nil {
		return m.Name
	}
	return nil
}

func (m *WikiCommitDetails) GetEmail() []byte {
	if m != nil {
		return m.Email
	}
	return nil
}

func (m *WikiCommitDetails) GetMessage() []byte {
	if m != nil {
		return m.Message
	}
	return nil
}

func (m *WikiCommitDetails) GetUserId() int32 {
	if m != nil {
		return m.UserId
	}
	return 0
}

func (m *WikiCommitDetails) GetUserName() []byte {
	if m != nil {
		return m.UserName
	}
	return nil
}

type WikiPageVersion struct {
	Commit               *GitCommit `protobuf:"bytes,1,opt,name=commit,proto3" json:"commit,omitempty"`
	Format               string     `protobuf:"bytes,2,opt,name=format,proto3" json:"format,omitempty"`
	XXX_NoUnkeyedLiteral struct{}   `json:"-"`
	XXX_unrecognized     []byte     `json:"-"`
	XXX_sizecache        int32      `json:"-"`
}

func (m *WikiPageVersion) Reset()         { *m = WikiPageVersion{} }
func (m *WikiPageVersion) String() string { return proto.CompactTextString(m) }
func (*WikiPageVersion) ProtoMessage()    {}
func (*WikiPageVersion) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{1}
}

func (m *WikiPageVersion) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiPageVersion.Unmarshal(m, b)
}
func (m *WikiPageVersion) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiPageVersion.Marshal(b, m, deterministic)
}
func (m *WikiPageVersion) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiPageVersion.Merge(m, src)
}
func (m *WikiPageVersion) XXX_Size() int {
	return xxx_messageInfo_WikiPageVersion.Size(m)
}
func (m *WikiPageVersion) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiPageVersion.DiscardUnknown(m)
}

var xxx_messageInfo_WikiPageVersion proto.InternalMessageInfo

func (m *WikiPageVersion) GetCommit() *GitCommit {
	if m != nil {
		return m.Commit
	}
	return nil
}

func (m *WikiPageVersion) GetFormat() string {
	if m != nil {
		return m.Format
	}
	return ""
}

type WikiPage struct {
	// These fields are only present in the first message of a WikiPage stream
	Version    *WikiPageVersion `protobuf:"bytes,1,opt,name=version,proto3" json:"version,omitempty"`
	Format     string           `protobuf:"bytes,2,opt,name=format,proto3" json:"format,omitempty"`
	Title      []byte           `protobuf:"bytes,3,opt,name=title,proto3" json:"title,omitempty"`
	UrlPath    string           `protobuf:"bytes,4,opt,name=url_path,json=urlPath,proto3" json:"url_path,omitempty"`
	Path       []byte           `protobuf:"bytes,5,opt,name=path,proto3" json:"path,omitempty"`
	Name       []byte           `protobuf:"bytes,6,opt,name=name,proto3" json:"name,omitempty"`
	Historical bool             `protobuf:"varint,7,opt,name=historical,proto3" json:"historical,omitempty"`
	// This field is present in all messages of a WikiPage stream
	RawData              []byte   `protobuf:"bytes,8,opt,name=raw_data,json=rawData,proto3" json:"raw_data,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *WikiPage) Reset()         { *m = WikiPage{} }
func (m *WikiPage) String() string { return proto.CompactTextString(m) }
func (*WikiPage) ProtoMessage()    {}
func (*WikiPage) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{2}
}

func (m *WikiPage) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiPage.Unmarshal(m, b)
}
func (m *WikiPage) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiPage.Marshal(b, m, deterministic)
}
func (m *WikiPage) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiPage.Merge(m, src)
}
func (m *WikiPage) XXX_Size() int {
	return xxx_messageInfo_WikiPage.Size(m)
}
func (m *WikiPage) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiPage.DiscardUnknown(m)
}

var xxx_messageInfo_WikiPage proto.InternalMessageInfo

func (m *WikiPage) GetVersion() *WikiPageVersion {
	if m != nil {
		return m.Version
	}
	return nil
}

func (m *WikiPage) GetFormat() string {
	if m != nil {
		return m.Format
	}
	return ""
}

func (m *WikiPage) GetTitle() []byte {
	if m != nil {
		return m.Title
	}
	return nil
}

func (m *WikiPage) GetUrlPath() string {
	if m != nil {
		return m.UrlPath
	}
	return ""
}

func (m *WikiPage) GetPath() []byte {
	if m != nil {
		return m.Path
	}
	return nil
}

func (m *WikiPage) GetName() []byte {
	if m != nil {
		return m.Name
	}
	return nil
}

func (m *WikiPage) GetHistorical() bool {
	if m != nil {
		return m.Historical
	}
	return false
}

func (m *WikiPage) GetRawData() []byte {
	if m != nil {
		return m.RawData
	}
	return nil
}

// This message is sent in a stream because the 'content' field may be large.
type WikiWritePageRequest struct {
	// These following fields are only present in the first message.
	Repository    *Repository        `protobuf:"bytes,1,opt,name=repository,proto3" json:"repository,omitempty"`
	Name          []byte             `protobuf:"bytes,2,opt,name=name,proto3" json:"name,omitempty"`
	Format        string             `protobuf:"bytes,3,opt,name=format,proto3" json:"format,omitempty"`
	CommitDetails *WikiCommitDetails `protobuf:"bytes,4,opt,name=commit_details,json=commitDetails,proto3" json:"commit_details,omitempty"`
	// This field is present in all messages.
	Content              []byte   `protobuf:"bytes,5,opt,name=content,proto3" json:"content,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *WikiWritePageRequest) Reset()         { *m = WikiWritePageRequest{} }
func (m *WikiWritePageRequest) String() string { return proto.CompactTextString(m) }
func (*WikiWritePageRequest) ProtoMessage()    {}
func (*WikiWritePageRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{3}
}

func (m *WikiWritePageRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiWritePageRequest.Unmarshal(m, b)
}
func (m *WikiWritePageRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiWritePageRequest.Marshal(b, m, deterministic)
}
func (m *WikiWritePageRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiWritePageRequest.Merge(m, src)
}
func (m *WikiWritePageRequest) XXX_Size() int {
	return xxx_messageInfo_WikiWritePageRequest.Size(m)
}
func (m *WikiWritePageRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiWritePageRequest.DiscardUnknown(m)
}

var xxx_messageInfo_WikiWritePageRequest proto.InternalMessageInfo

func (m *WikiWritePageRequest) GetRepository() *Repository {
	if m != nil {
		return m.Repository
	}
	return nil
}

func (m *WikiWritePageRequest) GetName() []byte {
	if m != nil {
		return m.Name
	}
	return nil
}

func (m *WikiWritePageRequest) GetFormat() string {
	if m != nil {
		return m.Format
	}
	return ""
}

func (m *WikiWritePageRequest) GetCommitDetails() *WikiCommitDetails {
	if m != nil {
		return m.CommitDetails
	}
	return nil
}

func (m *WikiWritePageRequest) GetContent() []byte {
	if m != nil {
		return m.Content
	}
	return nil
}

type WikiWritePageResponse struct {
	DuplicateError       []byte   `protobuf:"bytes,1,opt,name=duplicate_error,json=duplicateError,proto3" json:"duplicate_error,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *WikiWritePageResponse) Reset()         { *m = WikiWritePageResponse{} }
func (m *WikiWritePageResponse) String() string { return proto.CompactTextString(m) }
func (*WikiWritePageResponse) ProtoMessage()    {}
func (*WikiWritePageResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{4}
}

func (m *WikiWritePageResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiWritePageResponse.Unmarshal(m, b)
}
func (m *WikiWritePageResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiWritePageResponse.Marshal(b, m, deterministic)
}
func (m *WikiWritePageResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiWritePageResponse.Merge(m, src)
}
func (m *WikiWritePageResponse) XXX_Size() int {
	return xxx_messageInfo_WikiWritePageResponse.Size(m)
}
func (m *WikiWritePageResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiWritePageResponse.DiscardUnknown(m)
}

var xxx_messageInfo_WikiWritePageResponse proto.InternalMessageInfo

func (m *WikiWritePageResponse) GetDuplicateError() []byte {
	if m != nil {
		return m.DuplicateError
	}
	return nil
}

type WikiUpdatePageRequest struct {
	// There fields are only present in the first message of the stream
	Repository    *Repository        `protobuf:"bytes,1,opt,name=repository,proto3" json:"repository,omitempty"`
	PagePath      []byte             `protobuf:"bytes,2,opt,name=page_path,json=pagePath,proto3" json:"page_path,omitempty"`
	Title         []byte             `protobuf:"bytes,3,opt,name=title,proto3" json:"title,omitempty"`
	Format        string             `protobuf:"bytes,4,opt,name=format,proto3" json:"format,omitempty"`
	CommitDetails *WikiCommitDetails `protobuf:"bytes,5,opt,name=commit_details,json=commitDetails,proto3" json:"commit_details,omitempty"`
	// This field is present in all messages
	Content              []byte   `protobuf:"bytes,6,opt,name=content,proto3" json:"content,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *WikiUpdatePageRequest) Reset()         { *m = WikiUpdatePageRequest{} }
func (m *WikiUpdatePageRequest) String() string { return proto.CompactTextString(m) }
func (*WikiUpdatePageRequest) ProtoMessage()    {}
func (*WikiUpdatePageRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{5}
}

func (m *WikiUpdatePageRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiUpdatePageRequest.Unmarshal(m, b)
}
func (m *WikiUpdatePageRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiUpdatePageRequest.Marshal(b, m, deterministic)
}
func (m *WikiUpdatePageRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiUpdatePageRequest.Merge(m, src)
}
func (m *WikiUpdatePageRequest) XXX_Size() int {
	return xxx_messageInfo_WikiUpdatePageRequest.Size(m)
}
func (m *WikiUpdatePageRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiUpdatePageRequest.DiscardUnknown(m)
}

var xxx_messageInfo_WikiUpdatePageRequest proto.InternalMessageInfo

func (m *WikiUpdatePageRequest) GetRepository() *Repository {
	if m != nil {
		return m.Repository
	}
	return nil
}

func (m *WikiUpdatePageRequest) GetPagePath() []byte {
	if m != nil {
		return m.PagePath
	}
	return nil
}

func (m *WikiUpdatePageRequest) GetTitle() []byte {
	if m != nil {
		return m.Title
	}
	return nil
}

func (m *WikiUpdatePageRequest) GetFormat() string {
	if m != nil {
		return m.Format
	}
	return ""
}

func (m *WikiUpdatePageRequest) GetCommitDetails() *WikiCommitDetails {
	if m != nil {
		return m.CommitDetails
	}
	return nil
}

func (m *WikiUpdatePageRequest) GetContent() []byte {
	if m != nil {
		return m.Content
	}
	return nil
}

type WikiUpdatePageResponse struct {
	Error                []byte   `protobuf:"bytes,1,opt,name=error,proto3" json:"error,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *WikiUpdatePageResponse) Reset()         { *m = WikiUpdatePageResponse{} }
func (m *WikiUpdatePageResponse) String() string { return proto.CompactTextString(m) }
func (*WikiUpdatePageResponse) ProtoMessage()    {}
func (*WikiUpdatePageResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{6}
}

func (m *WikiUpdatePageResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiUpdatePageResponse.Unmarshal(m, b)
}
func (m *WikiUpdatePageResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiUpdatePageResponse.Marshal(b, m, deterministic)
}
func (m *WikiUpdatePageResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiUpdatePageResponse.Merge(m, src)
}
func (m *WikiUpdatePageResponse) XXX_Size() int {
	return xxx_messageInfo_WikiUpdatePageResponse.Size(m)
}
func (m *WikiUpdatePageResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiUpdatePageResponse.DiscardUnknown(m)
}

var xxx_messageInfo_WikiUpdatePageResponse proto.InternalMessageInfo

func (m *WikiUpdatePageResponse) GetError() []byte {
	if m != nil {
		return m.Error
	}
	return nil
}

type WikiFindPageRequest struct {
	Repository           *Repository `protobuf:"bytes,1,opt,name=repository,proto3" json:"repository,omitempty"`
	Title                []byte      `protobuf:"bytes,2,opt,name=title,proto3" json:"title,omitempty"`
	Revision             []byte      `protobuf:"bytes,3,opt,name=revision,proto3" json:"revision,omitempty"`
	Directory            []byte      `protobuf:"bytes,4,opt,name=directory,proto3" json:"directory,omitempty"`
	XXX_NoUnkeyedLiteral struct{}    `json:"-"`
	XXX_unrecognized     []byte      `json:"-"`
	XXX_sizecache        int32       `json:"-"`
}

func (m *WikiFindPageRequest) Reset()         { *m = WikiFindPageRequest{} }
func (m *WikiFindPageRequest) String() string { return proto.CompactTextString(m) }
func (*WikiFindPageRequest) ProtoMessage()    {}
func (*WikiFindPageRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{7}
}

func (m *WikiFindPageRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiFindPageRequest.Unmarshal(m, b)
}
func (m *WikiFindPageRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiFindPageRequest.Marshal(b, m, deterministic)
}
func (m *WikiFindPageRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiFindPageRequest.Merge(m, src)
}
func (m *WikiFindPageRequest) XXX_Size() int {
	return xxx_messageInfo_WikiFindPageRequest.Size(m)
}
func (m *WikiFindPageRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiFindPageRequest.DiscardUnknown(m)
}

var xxx_messageInfo_WikiFindPageRequest proto.InternalMessageInfo

func (m *WikiFindPageRequest) GetRepository() *Repository {
	if m != nil {
		return m.Repository
	}
	return nil
}

func (m *WikiFindPageRequest) GetTitle() []byte {
	if m != nil {
		return m.Title
	}
	return nil
}

func (m *WikiFindPageRequest) GetRevision() []byte {
	if m != nil {
		return m.Revision
	}
	return nil
}

func (m *WikiFindPageRequest) GetDirectory() []byte {
	if m != nil {
		return m.Directory
	}
	return nil
}

// WikiFindPageResponse is a stream because we need multiple WikiPage
// messages to send the raw_data field.
type WikiFindPageResponse struct {
	Page                 *WikiPage `protobuf:"bytes,1,opt,name=page,proto3" json:"page,omitempty"`
	XXX_NoUnkeyedLiteral struct{}  `json:"-"`
	XXX_unrecognized     []byte    `json:"-"`
	XXX_sizecache        int32     `json:"-"`
}

func (m *WikiFindPageResponse) Reset()         { *m = WikiFindPageResponse{} }
func (m *WikiFindPageResponse) String() string { return proto.CompactTextString(m) }
func (*WikiFindPageResponse) ProtoMessage()    {}
func (*WikiFindPageResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{8}
}

func (m *WikiFindPageResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiFindPageResponse.Unmarshal(m, b)
}
func (m *WikiFindPageResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiFindPageResponse.Marshal(b, m, deterministic)
}
func (m *WikiFindPageResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiFindPageResponse.Merge(m, src)
}
func (m *WikiFindPageResponse) XXX_Size() int {
	return xxx_messageInfo_WikiFindPageResponse.Size(m)
}
func (m *WikiFindPageResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiFindPageResponse.DiscardUnknown(m)
}

var xxx_messageInfo_WikiFindPageResponse proto.InternalMessageInfo

func (m *WikiFindPageResponse) GetPage() *WikiPage {
	if m != nil {
		return m.Page
	}
	return nil
}

type WikiGetAllPagesRequest struct {
	Repository *Repository `protobuf:"bytes,1,opt,name=repository,proto3" json:"repository,omitempty"`
	// Passing 0 means no limit is applied
	Limit                uint32                        `protobuf:"varint,2,opt,name=limit,proto3" json:"limit,omitempty"`
	DirectionDesc        bool                          `protobuf:"varint,3,opt,name=direction_desc,json=directionDesc,proto3" json:"direction_desc,omitempty"`
	Sort                 WikiGetAllPagesRequest_SortBy `protobuf:"varint,4,opt,name=sort,proto3,enum=gitaly.WikiGetAllPagesRequest_SortBy" json:"sort,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                      `json:"-"`
	XXX_unrecognized     []byte                        `json:"-"`
	XXX_sizecache        int32                         `json:"-"`
}

func (m *WikiGetAllPagesRequest) Reset()         { *m = WikiGetAllPagesRequest{} }
func (m *WikiGetAllPagesRequest) String() string { return proto.CompactTextString(m) }
func (*WikiGetAllPagesRequest) ProtoMessage()    {}
func (*WikiGetAllPagesRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{9}
}

func (m *WikiGetAllPagesRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiGetAllPagesRequest.Unmarshal(m, b)
}
func (m *WikiGetAllPagesRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiGetAllPagesRequest.Marshal(b, m, deterministic)
}
func (m *WikiGetAllPagesRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiGetAllPagesRequest.Merge(m, src)
}
func (m *WikiGetAllPagesRequest) XXX_Size() int {
	return xxx_messageInfo_WikiGetAllPagesRequest.Size(m)
}
func (m *WikiGetAllPagesRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiGetAllPagesRequest.DiscardUnknown(m)
}

var xxx_messageInfo_WikiGetAllPagesRequest proto.InternalMessageInfo

func (m *WikiGetAllPagesRequest) GetRepository() *Repository {
	if m != nil {
		return m.Repository
	}
	return nil
}

func (m *WikiGetAllPagesRequest) GetLimit() uint32 {
	if m != nil {
		return m.Limit
	}
	return 0
}

func (m *WikiGetAllPagesRequest) GetDirectionDesc() bool {
	if m != nil {
		return m.DirectionDesc
	}
	return false
}

func (m *WikiGetAllPagesRequest) GetSort() WikiGetAllPagesRequest_SortBy {
	if m != nil {
		return m.Sort
	}
	return WikiGetAllPagesRequest_TITLE
}

// The WikiGetAllPagesResponse stream is a concatenation of WikiPage streams
type WikiGetAllPagesResponse struct {
	Page *WikiPage `protobuf:"bytes,1,opt,name=page,proto3" json:"page,omitempty"`
	// When end_of_page is true it signals a change of page for the next Response message (if any)
	EndOfPage            bool     `protobuf:"varint,2,opt,name=end_of_page,json=endOfPage,proto3" json:"end_of_page,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *WikiGetAllPagesResponse) Reset()         { *m = WikiGetAllPagesResponse{} }
func (m *WikiGetAllPagesResponse) String() string { return proto.CompactTextString(m) }
func (*WikiGetAllPagesResponse) ProtoMessage()    {}
func (*WikiGetAllPagesResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{10}
}

func (m *WikiGetAllPagesResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiGetAllPagesResponse.Unmarshal(m, b)
}
func (m *WikiGetAllPagesResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiGetAllPagesResponse.Marshal(b, m, deterministic)
}
func (m *WikiGetAllPagesResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiGetAllPagesResponse.Merge(m, src)
}
func (m *WikiGetAllPagesResponse) XXX_Size() int {
	return xxx_messageInfo_WikiGetAllPagesResponse.Size(m)
}
func (m *WikiGetAllPagesResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiGetAllPagesResponse.DiscardUnknown(m)
}

var xxx_messageInfo_WikiGetAllPagesResponse proto.InternalMessageInfo

func (m *WikiGetAllPagesResponse) GetPage() *WikiPage {
	if m != nil {
		return m.Page
	}
	return nil
}

func (m *WikiGetAllPagesResponse) GetEndOfPage() bool {
	if m != nil {
		return m.EndOfPage
	}
	return false
}

type WikiListPagesRequest struct {
	Repository *Repository `protobuf:"bytes,1,opt,name=repository,proto3" json:"repository,omitempty"`
	// Passing 0 means no limit is applied
	Limit                uint32                      `protobuf:"varint,2,opt,name=limit,proto3" json:"limit,omitempty"`
	DirectionDesc        bool                        `protobuf:"varint,3,opt,name=direction_desc,json=directionDesc,proto3" json:"direction_desc,omitempty"`
	Sort                 WikiListPagesRequest_SortBy `protobuf:"varint,4,opt,name=sort,proto3,enum=gitaly.WikiListPagesRequest_SortBy" json:"sort,omitempty"`
	Offset               uint32                      `protobuf:"varint,5,opt,name=offset,proto3" json:"offset,omitempty"`
	XXX_NoUnkeyedLiteral struct{}                    `json:"-"`
	XXX_unrecognized     []byte                      `json:"-"`
	XXX_sizecache        int32                       `json:"-"`
}

func (m *WikiListPagesRequest) Reset()         { *m = WikiListPagesRequest{} }
func (m *WikiListPagesRequest) String() string { return proto.CompactTextString(m) }
func (*WikiListPagesRequest) ProtoMessage()    {}
func (*WikiListPagesRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{11}
}

func (m *WikiListPagesRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiListPagesRequest.Unmarshal(m, b)
}
func (m *WikiListPagesRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiListPagesRequest.Marshal(b, m, deterministic)
}
func (m *WikiListPagesRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiListPagesRequest.Merge(m, src)
}
func (m *WikiListPagesRequest) XXX_Size() int {
	return xxx_messageInfo_WikiListPagesRequest.Size(m)
}
func (m *WikiListPagesRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiListPagesRequest.DiscardUnknown(m)
}

var xxx_messageInfo_WikiListPagesRequest proto.InternalMessageInfo

func (m *WikiListPagesRequest) GetRepository() *Repository {
	if m != nil {
		return m.Repository
	}
	return nil
}

func (m *WikiListPagesRequest) GetLimit() uint32 {
	if m != nil {
		return m.Limit
	}
	return 0
}

func (m *WikiListPagesRequest) GetDirectionDesc() bool {
	if m != nil {
		return m.DirectionDesc
	}
	return false
}

func (m *WikiListPagesRequest) GetSort() WikiListPagesRequest_SortBy {
	if m != nil {
		return m.Sort
	}
	return WikiListPagesRequest_TITLE
}

func (m *WikiListPagesRequest) GetOffset() uint32 {
	if m != nil {
		return m.Offset
	}
	return 0
}

// The WikiListPagesResponse stream is a concatenation of WikiPage streams without content
type WikiListPagesResponse struct {
	Page                 *WikiPage `protobuf:"bytes,1,opt,name=page,proto3" json:"page,omitempty"`
	XXX_NoUnkeyedLiteral struct{}  `json:"-"`
	XXX_unrecognized     []byte    `json:"-"`
	XXX_sizecache        int32     `json:"-"`
}

func (m *WikiListPagesResponse) Reset()         { *m = WikiListPagesResponse{} }
func (m *WikiListPagesResponse) String() string { return proto.CompactTextString(m) }
func (*WikiListPagesResponse) ProtoMessage()    {}
func (*WikiListPagesResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_5c56f90469cec0af, []int{12}
}

func (m *WikiListPagesResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_WikiListPagesResponse.Unmarshal(m, b)
}
func (m *WikiListPagesResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_WikiListPagesResponse.Marshal(b, m, deterministic)
}
func (m *WikiListPagesResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_WikiListPagesResponse.Merge(m, src)
}
func (m *WikiListPagesResponse) XXX_Size() int {
	return xxx_messageInfo_WikiListPagesResponse.Size(m)
}
func (m *WikiListPagesResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_WikiListPagesResponse.DiscardUnknown(m)
}

var xxx_messageInfo_WikiListPagesResponse proto.InternalMessageInfo

func (m *WikiListPagesResponse) GetPage() *WikiPage {
	if m != nil {
		return m.Page
	}
	return nil
}

func init() {
	proto.RegisterEnum("gitaly.WikiGetAllPagesRequest_SortBy", WikiGetAllPagesRequest_SortBy_name, WikiGetAllPagesRequest_SortBy_value)
	proto.RegisterEnum("gitaly.WikiListPagesRequest_SortBy", WikiListPagesRequest_SortBy_name, WikiListPagesRequest_SortBy_value)
	proto.RegisterType((*WikiCommitDetails)(nil), "gitaly.WikiCommitDetails")
	proto.RegisterType((*WikiPageVersion)(nil), "gitaly.WikiPageVersion")
	proto.RegisterType((*WikiPage)(nil), "gitaly.WikiPage")
	proto.RegisterType((*WikiWritePageRequest)(nil), "gitaly.WikiWritePageRequest")
	proto.RegisterType((*WikiWritePageResponse)(nil), "gitaly.WikiWritePageResponse")
	proto.RegisterType((*WikiUpdatePageRequest)(nil), "gitaly.WikiUpdatePageRequest")
	proto.RegisterType((*WikiUpdatePageResponse)(nil), "gitaly.WikiUpdatePageResponse")
	proto.RegisterType((*WikiFindPageRequest)(nil), "gitaly.WikiFindPageRequest")
	proto.RegisterType((*WikiFindPageResponse)(nil), "gitaly.WikiFindPageResponse")
	proto.RegisterType((*WikiGetAllPagesRequest)(nil), "gitaly.WikiGetAllPagesRequest")
	proto.RegisterType((*WikiGetAllPagesResponse)(nil), "gitaly.WikiGetAllPagesResponse")
	proto.RegisterType((*WikiListPagesRequest)(nil), "gitaly.WikiListPagesRequest")
	proto.RegisterType((*WikiListPagesResponse)(nil), "gitaly.WikiListPagesResponse")
}

func init() { proto.RegisterFile("wiki.proto", fileDescriptor_5c56f90469cec0af) }

var fileDescriptor_5c56f90469cec0af = []byte{
	// 904 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xcc, 0x96, 0x41, 0x6f, 0x1b, 0x45,
	0x14, 0xc7, 0x59, 0xc7, 0x5e, 0xaf, 0x5f, 0x62, 0x37, 0x1d, 0x4a, 0xe3, 0x3a, 0x21, 0x44, 0x5b,
	0x2a, 0x8c, 0x04, 0x36, 0x35, 0x95, 0x00, 0x09, 0xa4, 0x26, 0x4d, 0xa8, 0x2a, 0x55, 0x50, 0x4d,
	0x4c, 0x2b, 0x10, 0xd2, 0x6a, 0xb2, 0x3b, 0x76, 0x46, 0x5d, 0xef, 0x2c, 0x33, 0xe3, 0x44, 0x39,
	0x73, 0xe6, 0x0c, 0x37, 0xbe, 0x0d, 0x17, 0xbe, 0x00, 0x5f, 0x81, 0x13, 0x57, 0xc4, 0x09, 0xcd,
	0xcc, 0x7a, 0xbd, 0xbb, 0x6e, 0x80, 0xa2, 0x1c, 0x7a, 0xdb, 0xf7, 0xde, 0xcc, 0x9b, 0xf7, 0xff,
	0xcd, 0x9b, 0xa7, 0x05, 0x38, 0x67, 0xcf, 0xd9, 0x20, 0x15, 0x5c, 0x71, 0xe4, 0x4e, 0x99, 0x22,
	0xf1, 0x45, 0x0f, 0x62, 0x96, 0x28, 0xeb, 0xeb, 0x6d, 0xc8, 0x53, 0x22, 0x68, 0x64, 0x2d, 0xff,
	0x07, 0x07, 0xae, 0x3f, 0x63, 0xcf, 0xd9, 0x03, 0x3e, 0x9b, 0x31, 0x75, 0x48, 0x15, 0x61, 0xb1,
	0x44, 0x08, 0xea, 0x09, 0x99, 0xd1, 0xae, 0xb3, 0xe7, 0xf4, 0x37, 0xb0, 0xf9, 0x46, 0x37, 0xa0,
	0x41, 0x67, 0x84, 0xc5, 0xdd, 0x9a, 0x71, 0x5a, 0x03, 0x75, 0xa1, 0x39, 0xa3, 0x52, 0x92, 0x29,
	0xed, 0xae, 0x19, 0xff, 0xc2, 0x44, 0x5b, 0xd0, 0x9c, 0x4b, 0x2a, 0x02, 0x16, 0x75, 0xeb, 0x7b,
	0x4e, 0xbf, 0x81, 0x5d, 0x6d, 0x3e, 0x8a, 0xd0, 0x36, 0xb4, 0x4c, 0xc0, 0x9c, 0xd0, 0x30, 0x9b,
	0x3c, 0xed, 0xf8, 0x82, 0xcc, 0xa8, 0x3f, 0x86, 0x6b, 0xba, 0x9c, 0x27, 0x64, 0x4a, 0x9f, 0x52,
	0x21, 0x19, 0x4f, 0xd0, 0xbb, 0xe0, 0x86, 0xa6, 0x3a, 0x53, 0xce, 0xfa, 0xe8, 0xfa, 0xc0, 0xaa,
	0x1a, 0x3c, 0x64, 0xca, 0x96, 0x8d, 0xb3, 0x05, 0xe8, 0x26, 0xb8, 0x13, 0x2e, 0x66, 0x44, 0x99,
	0x22, 0x5b, 0x38, 0xb3, 0xfc, 0xdf, 0x1d, 0xf0, 0x16, 0x69, 0xd1, 0x5d, 0x68, 0x9e, 0xd9, 0xd4,
	0x59, 0xc2, 0xad, 0x45, 0xc2, 0xca, 0xc9, 0x78, 0xb1, 0xee, 0xb2, 0xbc, 0x9a, 0x89, 0x62, 0x2a,
	0x5e, 0x68, 0xb7, 0x06, 0xba, 0x05, 0xde, 0x5c, 0xc4, 0x41, 0x4a, 0xd4, 0xa9, 0x91, 0xde, 0xc2,
	0xcd, 0xb9, 0x88, 0x9f, 0x10, 0x75, 0xaa, 0xc1, 0x1a, 0xb7, 0x95, 0x6d, 0xbe, 0x73, 0xd8, 0x6e,
	0x01, 0xf6, 0x2e, 0xc0, 0x29, 0x93, 0x8a, 0x0b, 0x16, 0x92, 0xb8, 0xdb, 0xdc, 0x73, 0xfa, 0x1e,
	0x2e, 0x78, 0xf4, 0x11, 0x82, 0x9c, 0x07, 0x11, 0x51, 0xa4, 0xeb, 0x59, 0xee, 0x82, 0x9c, 0x1f,
	0x12, 0x45, 0xfc, 0xdf, 0x1c, 0xb8, 0xa1, 0x85, 0x3c, 0x13, 0x4c, 0x51, 0xad, 0x06, 0xd3, 0xef,
	0xe6, 0x54, 0x2a, 0xf4, 0x31, 0x80, 0xa0, 0x29, 0x97, 0x4c, 0x71, 0x71, 0x91, 0x49, 0x47, 0x0b,
	0xe9, 0x38, 0x8f, 0x1c, 0xd4, 0x7f, 0xfa, 0xe5, 0x3d, 0x07, 0x17, 0xd6, 0xe6, 0x15, 0xd6, 0x0a,
	0x15, 0x2e, 0x91, 0xac, 0x95, 0x90, 0xdc, 0x87, 0x8e, 0xbd, 0x8c, 0x20, 0xb2, 0xcd, 0x64, 0x10,
	0xac, 0x8f, 0x6e, 0x15, 0x21, 0x97, 0xba, 0x0d, 0xb7, 0xc3, 0x52, 0xf3, 0x75, 0xa1, 0x19, 0xf2,
	0x44, 0xd1, 0x44, 0x65, 0x98, 0x16, 0xa6, 0x7f, 0x1f, 0xde, 0xa8, 0x28, 0x93, 0x29, 0x4f, 0x24,
	0x45, 0xef, 0xc0, 0xb5, 0x68, 0x9e, 0xc6, 0x2c, 0x24, 0x8a, 0x06, 0x54, 0x08, 0x2e, 0xb2, 0xd6,
	0xed, 0xe4, 0xee, 0x23, 0xed, 0xf5, 0xff, 0x74, 0x6c, 0x8a, 0xaf, 0xd2, 0x88, 0x5c, 0x15, 0x9d,
	0x6d, 0x68, 0xa5, 0x64, 0x4a, 0xed, 0x7d, 0x5b, 0x44, 0x9e, 0x76, 0x98, 0x0b, 0x7f, 0x71, 0x87,
	0x2c, 0xe1, 0xd5, 0xff, 0x05, 0x5e, 0xe3, 0xff, 0xc3, 0x73, 0xcb, 0xf0, 0x06, 0x70, 0xb3, 0xaa,
	0x3c, 0xa3, 0xa7, 0x5f, 0x76, 0x81, 0x99, 0x35, 0xfc, 0x9f, 0x1d, 0x78, 0x5d, 0x6f, 0xf8, 0x9c,
	0x25, 0xd1, 0xd5, 0x80, 0xca, 0x59, 0xd4, 0x8a, 0x2c, 0x7a, 0xe0, 0x09, 0x7a, 0xc6, 0xcc, 0x7b,
	0xb4, 0x90, 0x72, 0x1b, 0xed, 0x40, 0x2b, 0x62, 0x82, 0x86, 0xe6, 0xa8, 0xba, 0x09, 0x2e, 0x1d,
	0xfe, 0xa7, 0xb6, 0xd1, 0x97, 0x05, 0x66, 0x7a, 0xde, 0xd6, 0x8f, 0x6c, 0x4a, 0xb3, 0xda, 0x36,
	0xab, 0xaf, 0x1b, 0x9b, 0xa8, 0xff, 0x87, 0x63, 0x81, 0x3c, 0xa4, 0x6a, 0x3f, 0x8e, 0x75, 0x40,
	0x5e, 0x89, 0xc4, 0x98, 0xe9, 0x51, 0xa5, 0x25, 0xb6, 0xb1, 0x35, 0xd0, 0x1d, 0xe8, 0xd8, 0xaa,
	0x19, 0x4f, 0x82, 0x88, 0xca, 0xd0, 0x08, 0xf5, 0x70, 0x3b, 0xf7, 0x1e, 0x52, 0x19, 0xa2, 0x4f,
	0xa0, 0x2e, 0xb9, 0xb0, 0x3d, 0xd1, 0x19, 0xdd, 0x29, 0xd6, 0xbd, 0x5a, 0xe4, 0xe0, 0x98, 0x0b,
	0x75, 0x70, 0x81, 0xcd, 0x16, 0xff, 0x36, 0xb8, 0xd6, 0x46, 0x2d, 0x68, 0x8c, 0x1f, 0x8d, 0x1f,
	0x1f, 0x6d, 0xbe, 0x86, 0x3a, 0x00, 0x0f, 0xf0, 0xd1, 0xfe, 0xf8, 0xe8, 0x30, 0xd8, 0x1f, 0x6f,
	0x3a, 0x7e, 0x00, 0x5b, 0x2b, 0xb9, 0x5e, 0x06, 0x19, 0xda, 0x85, 0x75, 0x9a, 0x44, 0x01, 0x9f,
	0x04, 0x66, 0x71, 0xcd, 0x88, 0x68, 0xd1, 0x24, 0xfa, 0x72, 0xa2, 0x57, 0xf9, 0xdf, 0xd7, 0xec,
	0x8d, 0x3c, 0x66, 0x52, 0xbd, 0x0a, 0x40, 0x3f, 0x2a, 0x01, 0xbd, 0x5d, 0x54, 0x55, 0x2d, 0xb1,
	0x84, 0x53, 0xbf, 0x4f, 0x3e, 0x99, 0x48, 0x6a, 0x27, 0x50, 0x1b, 0x67, 0xd6, 0x7f, 0xc3, 0xfc,
	0x99, 0x1d, 0x31, 0x85, 0x13, 0x5e, 0x06, 0xf2, 0xe8, 0xd7, 0x35, 0x58, 0xd7, 0xae, 0x63, 0x2a,
	0xce, 0x58, 0x48, 0xd1, 0x53, 0x68, 0x97, 0x86, 0x1e, 0xda, 0x29, 0x6e, 0xac, 0x4e, 0xf9, 0xde,
	0x9b, 0x97, 0x44, 0x6d, 0x0d, 0xbe, 0xfb, 0xd7, 0x8f, 0xfd, 0x9a, 0xe7, 0xf4, 0x1d, 0xf4, 0x35,
	0x74, 0xca, 0xf3, 0x00, 0x95, 0xb6, 0xae, 0x4c, 0xc8, 0xde, 0xee, 0x65, 0xe1, 0x95, 0xd4, 0xc7,
	0xb0, 0x51, 0x7c, 0x98, 0x68, 0xbb, 0xb8, 0xb3, 0x32, 0x4f, 0x7a, 0x3b, 0x2f, 0x0e, 0x96, 0x92,
	0xd6, 0x3e, 0x70, 0xd0, 0xb7, 0xf6, 0xcf, 0xa0, 0xd0, 0xbd, 0x68, 0xf7, 0x9f, 0x9f, 0x48, 0xef,
	0xad, 0x4b, 0xe3, 0x2b, 0xd9, 0x33, 0xca, 0xf9, 0xa5, 0x95, 0x29, 0x57, 0xbb, 0xa5, 0x4c, 0x79,
	0xe5, 0xa6, 0x97, 0x79, 0x0f, 0xee, 0x7d, 0x33, 0x9a, 0x32, 0x15, 0x93, 0x93, 0x41, 0xc8, 0x67,
	0x43, 0xfb, 0xf9, 0x3e, 0x17, 0xd3, 0xa1, 0xdd, 0x3f, 0x3c, 0xbb, 0x7b, 0x6f, 0x68, 0xfe, 0xc4,
	0x86, 0x53, 0x9e, 0xf9, 0xd2, 0x93, 0x13, 0xd7, 0xb8, 0x3e, 0xfc, 0x3b, 0x00, 0x00, 0xff, 0xff,
	0x6f, 0xde, 0xb9, 0xca, 0xcc, 0x09, 0x00, 0x00,
}

// Reference imports to suppress errors if they are not otherwise used.
var _ context.Context
var _ grpc.ClientConn

// This is a compile-time assertion to ensure that this generated file
// is compatible with the grpc package it is being compiled against.
const _ = grpc.SupportPackageIsVersion4

// WikiServiceClient is the client API for WikiService service.
//
// For semantics around ctx use and closing/ending streaming RPCs, please refer to https://godoc.org/google.golang.org/grpc#ClientConn.NewStream.
type WikiServiceClient interface {
	WikiWritePage(ctx context.Context, opts ...grpc.CallOption) (WikiService_WikiWritePageClient, error)
	WikiUpdatePage(ctx context.Context, opts ...grpc.CallOption) (WikiService_WikiUpdatePageClient, error)
	// WikiFindPage returns a stream because the page's raw_data field may be arbitrarily large.
	WikiFindPage(ctx context.Context, in *WikiFindPageRequest, opts ...grpc.CallOption) (WikiService_WikiFindPageClient, error)
	WikiGetAllPages(ctx context.Context, in *WikiGetAllPagesRequest, opts ...grpc.CallOption) (WikiService_WikiGetAllPagesClient, error)
	WikiListPages(ctx context.Context, in *WikiListPagesRequest, opts ...grpc.CallOption) (WikiService_WikiListPagesClient, error)
}

type wikiServiceClient struct {
	cc *grpc.ClientConn
}

func NewWikiServiceClient(cc *grpc.ClientConn) WikiServiceClient {
	return &wikiServiceClient{cc}
}

func (c *wikiServiceClient) WikiWritePage(ctx context.Context, opts ...grpc.CallOption) (WikiService_WikiWritePageClient, error) {
	stream, err := c.cc.NewStream(ctx, &_WikiService_serviceDesc.Streams[0], "/gitaly.WikiService/WikiWritePage", opts...)
	if err != nil {
		return nil, err
	}
	x := &wikiServiceWikiWritePageClient{stream}
	return x, nil
}

type WikiService_WikiWritePageClient interface {
	Send(*WikiWritePageRequest) error
	CloseAndRecv() (*WikiWritePageResponse, error)
	grpc.ClientStream
}

type wikiServiceWikiWritePageClient struct {
	grpc.ClientStream
}

func (x *wikiServiceWikiWritePageClient) Send(m *WikiWritePageRequest) error {
	return x.ClientStream.SendMsg(m)
}

func (x *wikiServiceWikiWritePageClient) CloseAndRecv() (*WikiWritePageResponse, error) {
	if err := x.ClientStream.CloseSend(); err != nil {
		return nil, err
	}
	m := new(WikiWritePageResponse)
	if err := x.ClientStream.RecvMsg(m); err != nil {
		return nil, err
	}
	return m, nil
}

func (c *wikiServiceClient) WikiUpdatePage(ctx context.Context, opts ...grpc.CallOption) (WikiService_WikiUpdatePageClient, error) {
	stream, err := c.cc.NewStream(ctx, &_WikiService_serviceDesc.Streams[1], "/gitaly.WikiService/WikiUpdatePage", opts...)
	if err != nil {
		return nil, err
	}
	x := &wikiServiceWikiUpdatePageClient{stream}
	return x, nil
}

type WikiService_WikiUpdatePageClient interface {
	Send(*WikiUpdatePageRequest) error
	CloseAndRecv() (*WikiUpdatePageResponse, error)
	grpc.ClientStream
}

type wikiServiceWikiUpdatePageClient struct {
	grpc.ClientStream
}

func (x *wikiServiceWikiUpdatePageClient) Send(m *WikiUpdatePageRequest) error {
	return x.ClientStream.SendMsg(m)
}

func (x *wikiServiceWikiUpdatePageClient) CloseAndRecv() (*WikiUpdatePageResponse, error) {
	if err := x.ClientStream.CloseSend(); err != nil {
		return nil, err
	}
	m := new(WikiUpdatePageResponse)
	if err := x.ClientStream.RecvMsg(m); err != nil {
		return nil, err
	}
	return m, nil
}

func (c *wikiServiceClient) WikiFindPage(ctx context.Context, in *WikiFindPageRequest, opts ...grpc.CallOption) (WikiService_WikiFindPageClient, error) {
	stream, err := c.cc.NewStream(ctx, &_WikiService_serviceDesc.Streams[2], "/gitaly.WikiService/WikiFindPage", opts...)
	if err != nil {
		return nil, err
	}
	x := &wikiServiceWikiFindPageClient{stream}
	if err := x.ClientStream.SendMsg(in); err != nil {
		return nil, err
	}
	if err := x.ClientStream.CloseSend(); err != nil {
		return nil, err
	}
	return x, nil
}

type WikiService_WikiFindPageClient interface {
	Recv() (*WikiFindPageResponse, error)
	grpc.ClientStream
}

type wikiServiceWikiFindPageClient struct {
	grpc.ClientStream
}

func (x *wikiServiceWikiFindPageClient) Recv() (*WikiFindPageResponse, error) {
	m := new(WikiFindPageResponse)
	if err := x.ClientStream.RecvMsg(m); err != nil {
		return nil, err
	}
	return m, nil
}

func (c *wikiServiceClient) WikiGetAllPages(ctx context.Context, in *WikiGetAllPagesRequest, opts ...grpc.CallOption) (WikiService_WikiGetAllPagesClient, error) {
	stream, err := c.cc.NewStream(ctx, &_WikiService_serviceDesc.Streams[3], "/gitaly.WikiService/WikiGetAllPages", opts...)
	if err != nil {
		return nil, err
	}
	x := &wikiServiceWikiGetAllPagesClient{stream}
	if err := x.ClientStream.SendMsg(in); err != nil {
		return nil, err
	}
	if err := x.ClientStream.CloseSend(); err != nil {
		return nil, err
	}
	return x, nil
}

type WikiService_WikiGetAllPagesClient interface {
	Recv() (*WikiGetAllPagesResponse, error)
	grpc.ClientStream
}

type wikiServiceWikiGetAllPagesClient struct {
	grpc.ClientStream
}

func (x *wikiServiceWikiGetAllPagesClient) Recv() (*WikiGetAllPagesResponse, error) {
	m := new(WikiGetAllPagesResponse)
	if err := x.ClientStream.RecvMsg(m); err != nil {
		return nil, err
	}
	return m, nil
}

func (c *wikiServiceClient) WikiListPages(ctx context.Context, in *WikiListPagesRequest, opts ...grpc.CallOption) (WikiService_WikiListPagesClient, error) {
	stream, err := c.cc.NewStream(ctx, &_WikiService_serviceDesc.Streams[4], "/gitaly.WikiService/WikiListPages", opts...)
	if err != nil {
		return nil, err
	}
	x := &wikiServiceWikiListPagesClient{stream}
	if err := x.ClientStream.SendMsg(in); err != nil {
		return nil, err
	}
	if err := x.ClientStream.CloseSend(); err != nil {
		return nil, err
	}
	return x, nil
}

type WikiService_WikiListPagesClient interface {
	Recv() (*WikiListPagesResponse, error)
	grpc.ClientStream
}

type wikiServiceWikiListPagesClient struct {
	grpc.ClientStream
}

func (x *wikiServiceWikiListPagesClient) Recv() (*WikiListPagesResponse, error) {
	m := new(WikiListPagesResponse)
	if err := x.ClientStream.RecvMsg(m); err != nil {
		return nil, err
	}
	return m, nil
}

// WikiServiceServer is the server API for WikiService service.
type WikiServiceServer interface {
	WikiWritePage(WikiService_WikiWritePageServer) error
	WikiUpdatePage(WikiService_WikiUpdatePageServer) error
	// WikiFindPage returns a stream because the page's raw_data field may be arbitrarily large.
	WikiFindPage(*WikiFindPageRequest, WikiService_WikiFindPageServer) error
	WikiGetAllPages(*WikiGetAllPagesRequest, WikiService_WikiGetAllPagesServer) error
	WikiListPages(*WikiListPagesRequest, WikiService_WikiListPagesServer) error
}

// UnimplementedWikiServiceServer can be embedded to have forward compatible implementations.
type UnimplementedWikiServiceServer struct {
}

func (*UnimplementedWikiServiceServer) WikiWritePage(srv WikiService_WikiWritePageServer) error {
	return status.Errorf(codes.Unimplemented, "method WikiWritePage not implemented")
}
func (*UnimplementedWikiServiceServer) WikiUpdatePage(srv WikiService_WikiUpdatePageServer) error {
	return status.Errorf(codes.Unimplemented, "method WikiUpdatePage not implemented")
}
func (*UnimplementedWikiServiceServer) WikiFindPage(req *WikiFindPageRequest, srv WikiService_WikiFindPageServer) error {
	return status.Errorf(codes.Unimplemented, "method WikiFindPage not implemented")
}
func (*UnimplementedWikiServiceServer) WikiGetAllPages(req *WikiGetAllPagesRequest, srv WikiService_WikiGetAllPagesServer) error {
	return status.Errorf(codes.Unimplemented, "method WikiGetAllPages not implemented")
}
func (*UnimplementedWikiServiceServer) WikiListPages(req *WikiListPagesRequest, srv WikiService_WikiListPagesServer) error {
	return status.Errorf(codes.Unimplemented, "method WikiListPages not implemented")
}

func RegisterWikiServiceServer(s *grpc.Server, srv WikiServiceServer) {
	s.RegisterService(&_WikiService_serviceDesc, srv)
}

func _WikiService_WikiWritePage_Handler(srv interface{}, stream grpc.ServerStream) error {
	return srv.(WikiServiceServer).WikiWritePage(&wikiServiceWikiWritePageServer{stream})
}

type WikiService_WikiWritePageServer interface {
	SendAndClose(*WikiWritePageResponse) error
	Recv() (*WikiWritePageRequest, error)
	grpc.ServerStream
}

type wikiServiceWikiWritePageServer struct {
	grpc.ServerStream
}

func (x *wikiServiceWikiWritePageServer) SendAndClose(m *WikiWritePageResponse) error {
	return x.ServerStream.SendMsg(m)
}

func (x *wikiServiceWikiWritePageServer) Recv() (*WikiWritePageRequest, error) {
	m := new(WikiWritePageRequest)
	if err := x.ServerStream.RecvMsg(m); err != nil {
		return nil, err
	}
	return m, nil
}

func _WikiService_WikiUpdatePage_Handler(srv interface{}, stream grpc.ServerStream) error {
	return srv.(WikiServiceServer).WikiUpdatePage(&wikiServiceWikiUpdatePageServer{stream})
}

type WikiService_WikiUpdatePageServer interface {
	SendAndClose(*WikiUpdatePageResponse) error
	Recv() (*WikiUpdatePageRequest, error)
	grpc.ServerStream
}

type wikiServiceWikiUpdatePageServer struct {
	grpc.ServerStream
}

func (x *wikiServiceWikiUpdatePageServer) SendAndClose(m *WikiUpdatePageResponse) error {
	return x.ServerStream.SendMsg(m)
}

func (x *wikiServiceWikiUpdatePageServer) Recv() (*WikiUpdatePageRequest, error) {
	m := new(WikiUpdatePageRequest)
	if err := x.ServerStream.RecvMsg(m); err != nil {
		return nil, err
	}
	return m, nil
}

func _WikiService_WikiFindPage_Handler(srv interface{}, stream grpc.ServerStream) error {
	m := new(WikiFindPageRequest)
	if err := stream.RecvMsg(m); err != nil {
		return err
	}
	return srv.(WikiServiceServer).WikiFindPage(m, &wikiServiceWikiFindPageServer{stream})
}

type WikiService_WikiFindPageServer interface {
	Send(*WikiFindPageResponse) error
	grpc.ServerStream
}

type wikiServiceWikiFindPageServer struct {
	grpc.ServerStream
}

func (x *wikiServiceWikiFindPageServer) Send(m *WikiFindPageResponse) error {
	return x.ServerStream.SendMsg(m)
}

func _WikiService_WikiGetAllPages_Handler(srv interface{}, stream grpc.ServerStream) error {
	m := new(WikiGetAllPagesRequest)
	if err := stream.RecvMsg(m); err != nil {
		return err
	}
	return srv.(WikiServiceServer).WikiGetAllPages(m, &wikiServiceWikiGetAllPagesServer{stream})
}

type WikiService_WikiGetAllPagesServer interface {
	Send(*WikiGetAllPagesResponse) error
	grpc.ServerStream
}

type wikiServiceWikiGetAllPagesServer struct {
	grpc.ServerStream
}

func (x *wikiServiceWikiGetAllPagesServer) Send(m *WikiGetAllPagesResponse) error {
	return x.ServerStream.SendMsg(m)
}

func _WikiService_WikiListPages_Handler(srv interface{}, stream grpc.ServerStream) error {
	m := new(WikiListPagesRequest)
	if err := stream.RecvMsg(m); err != nil {
		return err
	}
	return srv.(WikiServiceServer).WikiListPages(m, &wikiServiceWikiListPagesServer{stream})
}

type WikiService_WikiListPagesServer interface {
	Send(*WikiListPagesResponse) error
	grpc.ServerStream
}

type wikiServiceWikiListPagesServer struct {
	grpc.ServerStream
}

func (x *wikiServiceWikiListPagesServer) Send(m *WikiListPagesResponse) error {
	return x.ServerStream.SendMsg(m)
}

var _WikiService_serviceDesc = grpc.ServiceDesc{
	ServiceName: "gitaly.WikiService",
	HandlerType: (*WikiServiceServer)(nil),
	Methods:     []grpc.MethodDesc{},
	Streams: []grpc.StreamDesc{
		{
			StreamName:    "WikiWritePage",
			Handler:       _WikiService_WikiWritePage_Handler,
			ClientStreams: true,
		},
		{
			StreamName:    "WikiUpdatePage",
			Handler:       _WikiService_WikiUpdatePage_Handler,
			ClientStreams: true,
		},
		{
			StreamName:    "WikiFindPage",
			Handler:       _WikiService_WikiFindPage_Handler,
			ServerStreams: true,
		},
		{
			StreamName:    "WikiGetAllPages",
			Handler:       _WikiService_WikiGetAllPages_Handler,
			ServerStreams: true,
		},
		{
			StreamName:    "WikiListPages",
			Handler:       _WikiService_WikiListPages_Handler,
			ServerStreams: true,
		},
	},
	Metadata: "wiki.proto",
}
