// Code generated by protoc-gen-go. DO NOT EDIT.
// source: objectpool.proto

package gitalypb

import (
	context "context"
	fmt "fmt"
	proto "github.com/golang/protobuf/proto"
	grpc "google.golang.org/grpc"
	codes "google.golang.org/grpc/codes"
	status "google.golang.org/grpc/status"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

// Creates an object pool from the repository. The client is responsible for
// joining this pool later with this repository.
type CreateObjectPoolRequest struct {
	ObjectPool           *ObjectPool `protobuf:"bytes,1,opt,name=object_pool,json=objectPool,proto3" json:"object_pool,omitempty"`
	Origin               *Repository `protobuf:"bytes,2,opt,name=origin,proto3" json:"origin,omitempty"`
	XXX_NoUnkeyedLiteral struct{}    `json:"-"`
	XXX_unrecognized     []byte      `json:"-"`
	XXX_sizecache        int32       `json:"-"`
}

func (m *CreateObjectPoolRequest) Reset()         { *m = CreateObjectPoolRequest{} }
func (m *CreateObjectPoolRequest) String() string { return proto.CompactTextString(m) }
func (*CreateObjectPoolRequest) ProtoMessage()    {}
func (*CreateObjectPoolRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{0}
}

func (m *CreateObjectPoolRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_CreateObjectPoolRequest.Unmarshal(m, b)
}
func (m *CreateObjectPoolRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_CreateObjectPoolRequest.Marshal(b, m, deterministic)
}
func (m *CreateObjectPoolRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_CreateObjectPoolRequest.Merge(m, src)
}
func (m *CreateObjectPoolRequest) XXX_Size() int {
	return xxx_messageInfo_CreateObjectPoolRequest.Size(m)
}
func (m *CreateObjectPoolRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_CreateObjectPoolRequest.DiscardUnknown(m)
}

var xxx_messageInfo_CreateObjectPoolRequest proto.InternalMessageInfo

func (m *CreateObjectPoolRequest) GetObjectPool() *ObjectPool {
	if m != nil {
		return m.ObjectPool
	}
	return nil
}

func (m *CreateObjectPoolRequest) GetOrigin() *Repository {
	if m != nil {
		return m.Origin
	}
	return nil
}

type CreateObjectPoolResponse struct {
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *CreateObjectPoolResponse) Reset()         { *m = CreateObjectPoolResponse{} }
func (m *CreateObjectPoolResponse) String() string { return proto.CompactTextString(m) }
func (*CreateObjectPoolResponse) ProtoMessage()    {}
func (*CreateObjectPoolResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{1}
}

func (m *CreateObjectPoolResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_CreateObjectPoolResponse.Unmarshal(m, b)
}
func (m *CreateObjectPoolResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_CreateObjectPoolResponse.Marshal(b, m, deterministic)
}
func (m *CreateObjectPoolResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_CreateObjectPoolResponse.Merge(m, src)
}
func (m *CreateObjectPoolResponse) XXX_Size() int {
	return xxx_messageInfo_CreateObjectPoolResponse.Size(m)
}
func (m *CreateObjectPoolResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_CreateObjectPoolResponse.DiscardUnknown(m)
}

var xxx_messageInfo_CreateObjectPoolResponse proto.InternalMessageInfo

// Removes the directory from disk, caller is responsible for leaving the object
// pool before calling this RPC
type DeleteObjectPoolRequest struct {
	ObjectPool           *ObjectPool `protobuf:"bytes,1,opt,name=object_pool,json=objectPool,proto3" json:"object_pool,omitempty"`
	XXX_NoUnkeyedLiteral struct{}    `json:"-"`
	XXX_unrecognized     []byte      `json:"-"`
	XXX_sizecache        int32       `json:"-"`
}

func (m *DeleteObjectPoolRequest) Reset()         { *m = DeleteObjectPoolRequest{} }
func (m *DeleteObjectPoolRequest) String() string { return proto.CompactTextString(m) }
func (*DeleteObjectPoolRequest) ProtoMessage()    {}
func (*DeleteObjectPoolRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{2}
}

func (m *DeleteObjectPoolRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_DeleteObjectPoolRequest.Unmarshal(m, b)
}
func (m *DeleteObjectPoolRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_DeleteObjectPoolRequest.Marshal(b, m, deterministic)
}
func (m *DeleteObjectPoolRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_DeleteObjectPoolRequest.Merge(m, src)
}
func (m *DeleteObjectPoolRequest) XXX_Size() int {
	return xxx_messageInfo_DeleteObjectPoolRequest.Size(m)
}
func (m *DeleteObjectPoolRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_DeleteObjectPoolRequest.DiscardUnknown(m)
}

var xxx_messageInfo_DeleteObjectPoolRequest proto.InternalMessageInfo

func (m *DeleteObjectPoolRequest) GetObjectPool() *ObjectPool {
	if m != nil {
		return m.ObjectPool
	}
	return nil
}

type DeleteObjectPoolResponse struct {
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *DeleteObjectPoolResponse) Reset()         { *m = DeleteObjectPoolResponse{} }
func (m *DeleteObjectPoolResponse) String() string { return proto.CompactTextString(m) }
func (*DeleteObjectPoolResponse) ProtoMessage()    {}
func (*DeleteObjectPoolResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{3}
}

func (m *DeleteObjectPoolResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_DeleteObjectPoolResponse.Unmarshal(m, b)
}
func (m *DeleteObjectPoolResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_DeleteObjectPoolResponse.Marshal(b, m, deterministic)
}
func (m *DeleteObjectPoolResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_DeleteObjectPoolResponse.Merge(m, src)
}
func (m *DeleteObjectPoolResponse) XXX_Size() int {
	return xxx_messageInfo_DeleteObjectPoolResponse.Size(m)
}
func (m *DeleteObjectPoolResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_DeleteObjectPoolResponse.DiscardUnknown(m)
}

var xxx_messageInfo_DeleteObjectPoolResponse proto.InternalMessageInfo

type LinkRepositoryToObjectPoolRequest struct {
	ObjectPool           *ObjectPool `protobuf:"bytes,1,opt,name=object_pool,json=objectPool,proto3" json:"object_pool,omitempty"`
	Repository           *Repository `protobuf:"bytes,2,opt,name=repository,proto3" json:"repository,omitempty"`
	XXX_NoUnkeyedLiteral struct{}    `json:"-"`
	XXX_unrecognized     []byte      `json:"-"`
	XXX_sizecache        int32       `json:"-"`
}

func (m *LinkRepositoryToObjectPoolRequest) Reset()         { *m = LinkRepositoryToObjectPoolRequest{} }
func (m *LinkRepositoryToObjectPoolRequest) String() string { return proto.CompactTextString(m) }
func (*LinkRepositoryToObjectPoolRequest) ProtoMessage()    {}
func (*LinkRepositoryToObjectPoolRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{4}
}

func (m *LinkRepositoryToObjectPoolRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_LinkRepositoryToObjectPoolRequest.Unmarshal(m, b)
}
func (m *LinkRepositoryToObjectPoolRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_LinkRepositoryToObjectPoolRequest.Marshal(b, m, deterministic)
}
func (m *LinkRepositoryToObjectPoolRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_LinkRepositoryToObjectPoolRequest.Merge(m, src)
}
func (m *LinkRepositoryToObjectPoolRequest) XXX_Size() int {
	return xxx_messageInfo_LinkRepositoryToObjectPoolRequest.Size(m)
}
func (m *LinkRepositoryToObjectPoolRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_LinkRepositoryToObjectPoolRequest.DiscardUnknown(m)
}

var xxx_messageInfo_LinkRepositoryToObjectPoolRequest proto.InternalMessageInfo

func (m *LinkRepositoryToObjectPoolRequest) GetObjectPool() *ObjectPool {
	if m != nil {
		return m.ObjectPool
	}
	return nil
}

func (m *LinkRepositoryToObjectPoolRequest) GetRepository() *Repository {
	if m != nil {
		return m.Repository
	}
	return nil
}

type LinkRepositoryToObjectPoolResponse struct {
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *LinkRepositoryToObjectPoolResponse) Reset()         { *m = LinkRepositoryToObjectPoolResponse{} }
func (m *LinkRepositoryToObjectPoolResponse) String() string { return proto.CompactTextString(m) }
func (*LinkRepositoryToObjectPoolResponse) ProtoMessage()    {}
func (*LinkRepositoryToObjectPoolResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{5}
}

func (m *LinkRepositoryToObjectPoolResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_LinkRepositoryToObjectPoolResponse.Unmarshal(m, b)
}
func (m *LinkRepositoryToObjectPoolResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_LinkRepositoryToObjectPoolResponse.Marshal(b, m, deterministic)
}
func (m *LinkRepositoryToObjectPoolResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_LinkRepositoryToObjectPoolResponse.Merge(m, src)
}
func (m *LinkRepositoryToObjectPoolResponse) XXX_Size() int {
	return xxx_messageInfo_LinkRepositoryToObjectPoolResponse.Size(m)
}
func (m *LinkRepositoryToObjectPoolResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_LinkRepositoryToObjectPoolResponse.DiscardUnknown(m)
}

var xxx_messageInfo_LinkRepositoryToObjectPoolResponse proto.InternalMessageInfo

// This RPC doesn't require the ObjectPool as it will remove the alternates file
// from the pool participant. The caller is responsible no data loss occurs.
type UnlinkRepositoryFromObjectPoolRequest struct {
	Repository           *Repository `protobuf:"bytes,1,opt,name=repository,proto3" json:"repository,omitempty"`
	ObjectPool           *ObjectPool `protobuf:"bytes,2,opt,name=object_pool,json=objectPool,proto3" json:"object_pool,omitempty"`
	XXX_NoUnkeyedLiteral struct{}    `json:"-"`
	XXX_unrecognized     []byte      `json:"-"`
	XXX_sizecache        int32       `json:"-"`
}

func (m *UnlinkRepositoryFromObjectPoolRequest) Reset()         { *m = UnlinkRepositoryFromObjectPoolRequest{} }
func (m *UnlinkRepositoryFromObjectPoolRequest) String() string { return proto.CompactTextString(m) }
func (*UnlinkRepositoryFromObjectPoolRequest) ProtoMessage()    {}
func (*UnlinkRepositoryFromObjectPoolRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{6}
}

func (m *UnlinkRepositoryFromObjectPoolRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_UnlinkRepositoryFromObjectPoolRequest.Unmarshal(m, b)
}
func (m *UnlinkRepositoryFromObjectPoolRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_UnlinkRepositoryFromObjectPoolRequest.Marshal(b, m, deterministic)
}
func (m *UnlinkRepositoryFromObjectPoolRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_UnlinkRepositoryFromObjectPoolRequest.Merge(m, src)
}
func (m *UnlinkRepositoryFromObjectPoolRequest) XXX_Size() int {
	return xxx_messageInfo_UnlinkRepositoryFromObjectPoolRequest.Size(m)
}
func (m *UnlinkRepositoryFromObjectPoolRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_UnlinkRepositoryFromObjectPoolRequest.DiscardUnknown(m)
}

var xxx_messageInfo_UnlinkRepositoryFromObjectPoolRequest proto.InternalMessageInfo

func (m *UnlinkRepositoryFromObjectPoolRequest) GetRepository() *Repository {
	if m != nil {
		return m.Repository
	}
	return nil
}

func (m *UnlinkRepositoryFromObjectPoolRequest) GetObjectPool() *ObjectPool {
	if m != nil {
		return m.ObjectPool
	}
	return nil
}

type UnlinkRepositoryFromObjectPoolResponse struct {
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *UnlinkRepositoryFromObjectPoolResponse) Reset() {
	*m = UnlinkRepositoryFromObjectPoolResponse{}
}
func (m *UnlinkRepositoryFromObjectPoolResponse) String() string { return proto.CompactTextString(m) }
func (*UnlinkRepositoryFromObjectPoolResponse) ProtoMessage()    {}
func (*UnlinkRepositoryFromObjectPoolResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{7}
}

func (m *UnlinkRepositoryFromObjectPoolResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_UnlinkRepositoryFromObjectPoolResponse.Unmarshal(m, b)
}
func (m *UnlinkRepositoryFromObjectPoolResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_UnlinkRepositoryFromObjectPoolResponse.Marshal(b, m, deterministic)
}
func (m *UnlinkRepositoryFromObjectPoolResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_UnlinkRepositoryFromObjectPoolResponse.Merge(m, src)
}
func (m *UnlinkRepositoryFromObjectPoolResponse) XXX_Size() int {
	return xxx_messageInfo_UnlinkRepositoryFromObjectPoolResponse.Size(m)
}
func (m *UnlinkRepositoryFromObjectPoolResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_UnlinkRepositoryFromObjectPoolResponse.DiscardUnknown(m)
}

var xxx_messageInfo_UnlinkRepositoryFromObjectPoolResponse proto.InternalMessageInfo

type ReduplicateRepositoryRequest struct {
	Repository           *Repository `protobuf:"bytes,1,opt,name=repository,proto3" json:"repository,omitempty"`
	XXX_NoUnkeyedLiteral struct{}    `json:"-"`
	XXX_unrecognized     []byte      `json:"-"`
	XXX_sizecache        int32       `json:"-"`
}

func (m *ReduplicateRepositoryRequest) Reset()         { *m = ReduplicateRepositoryRequest{} }
func (m *ReduplicateRepositoryRequest) String() string { return proto.CompactTextString(m) }
func (*ReduplicateRepositoryRequest) ProtoMessage()    {}
func (*ReduplicateRepositoryRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{8}
}

func (m *ReduplicateRepositoryRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ReduplicateRepositoryRequest.Unmarshal(m, b)
}
func (m *ReduplicateRepositoryRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ReduplicateRepositoryRequest.Marshal(b, m, deterministic)
}
func (m *ReduplicateRepositoryRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ReduplicateRepositoryRequest.Merge(m, src)
}
func (m *ReduplicateRepositoryRequest) XXX_Size() int {
	return xxx_messageInfo_ReduplicateRepositoryRequest.Size(m)
}
func (m *ReduplicateRepositoryRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_ReduplicateRepositoryRequest.DiscardUnknown(m)
}

var xxx_messageInfo_ReduplicateRepositoryRequest proto.InternalMessageInfo

func (m *ReduplicateRepositoryRequest) GetRepository() *Repository {
	if m != nil {
		return m.Repository
	}
	return nil
}

type ReduplicateRepositoryResponse struct {
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *ReduplicateRepositoryResponse) Reset()         { *m = ReduplicateRepositoryResponse{} }
func (m *ReduplicateRepositoryResponse) String() string { return proto.CompactTextString(m) }
func (*ReduplicateRepositoryResponse) ProtoMessage()    {}
func (*ReduplicateRepositoryResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{9}
}

func (m *ReduplicateRepositoryResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ReduplicateRepositoryResponse.Unmarshal(m, b)
}
func (m *ReduplicateRepositoryResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ReduplicateRepositoryResponse.Marshal(b, m, deterministic)
}
func (m *ReduplicateRepositoryResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ReduplicateRepositoryResponse.Merge(m, src)
}
func (m *ReduplicateRepositoryResponse) XXX_Size() int {
	return xxx_messageInfo_ReduplicateRepositoryResponse.Size(m)
}
func (m *ReduplicateRepositoryResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_ReduplicateRepositoryResponse.DiscardUnknown(m)
}

var xxx_messageInfo_ReduplicateRepositoryResponse proto.InternalMessageInfo

type DisconnectGitAlternatesRequest struct {
	Repository           *Repository `protobuf:"bytes,1,opt,name=repository,proto3" json:"repository,omitempty"`
	XXX_NoUnkeyedLiteral struct{}    `json:"-"`
	XXX_unrecognized     []byte      `json:"-"`
	XXX_sizecache        int32       `json:"-"`
}

func (m *DisconnectGitAlternatesRequest) Reset()         { *m = DisconnectGitAlternatesRequest{} }
func (m *DisconnectGitAlternatesRequest) String() string { return proto.CompactTextString(m) }
func (*DisconnectGitAlternatesRequest) ProtoMessage()    {}
func (*DisconnectGitAlternatesRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{10}
}

func (m *DisconnectGitAlternatesRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_DisconnectGitAlternatesRequest.Unmarshal(m, b)
}
func (m *DisconnectGitAlternatesRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_DisconnectGitAlternatesRequest.Marshal(b, m, deterministic)
}
func (m *DisconnectGitAlternatesRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_DisconnectGitAlternatesRequest.Merge(m, src)
}
func (m *DisconnectGitAlternatesRequest) XXX_Size() int {
	return xxx_messageInfo_DisconnectGitAlternatesRequest.Size(m)
}
func (m *DisconnectGitAlternatesRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_DisconnectGitAlternatesRequest.DiscardUnknown(m)
}

var xxx_messageInfo_DisconnectGitAlternatesRequest proto.InternalMessageInfo

func (m *DisconnectGitAlternatesRequest) GetRepository() *Repository {
	if m != nil {
		return m.Repository
	}
	return nil
}

type DisconnectGitAlternatesResponse struct {
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *DisconnectGitAlternatesResponse) Reset()         { *m = DisconnectGitAlternatesResponse{} }
func (m *DisconnectGitAlternatesResponse) String() string { return proto.CompactTextString(m) }
func (*DisconnectGitAlternatesResponse) ProtoMessage()    {}
func (*DisconnectGitAlternatesResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{11}
}

func (m *DisconnectGitAlternatesResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_DisconnectGitAlternatesResponse.Unmarshal(m, b)
}
func (m *DisconnectGitAlternatesResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_DisconnectGitAlternatesResponse.Marshal(b, m, deterministic)
}
func (m *DisconnectGitAlternatesResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_DisconnectGitAlternatesResponse.Merge(m, src)
}
func (m *DisconnectGitAlternatesResponse) XXX_Size() int {
	return xxx_messageInfo_DisconnectGitAlternatesResponse.Size(m)
}
func (m *DisconnectGitAlternatesResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_DisconnectGitAlternatesResponse.DiscardUnknown(m)
}

var xxx_messageInfo_DisconnectGitAlternatesResponse proto.InternalMessageInfo

type FetchIntoObjectPoolRequest struct {
	Origin               *Repository `protobuf:"bytes,1,opt,name=origin,proto3" json:"origin,omitempty"`
	ObjectPool           *ObjectPool `protobuf:"bytes,2,opt,name=object_pool,json=objectPool,proto3" json:"object_pool,omitempty"`
	Repack               bool        `protobuf:"varint,3,opt,name=repack,proto3" json:"repack,omitempty"`
	XXX_NoUnkeyedLiteral struct{}    `json:"-"`
	XXX_unrecognized     []byte      `json:"-"`
	XXX_sizecache        int32       `json:"-"`
}

func (m *FetchIntoObjectPoolRequest) Reset()         { *m = FetchIntoObjectPoolRequest{} }
func (m *FetchIntoObjectPoolRequest) String() string { return proto.CompactTextString(m) }
func (*FetchIntoObjectPoolRequest) ProtoMessage()    {}
func (*FetchIntoObjectPoolRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{12}
}

func (m *FetchIntoObjectPoolRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_FetchIntoObjectPoolRequest.Unmarshal(m, b)
}
func (m *FetchIntoObjectPoolRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_FetchIntoObjectPoolRequest.Marshal(b, m, deterministic)
}
func (m *FetchIntoObjectPoolRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_FetchIntoObjectPoolRequest.Merge(m, src)
}
func (m *FetchIntoObjectPoolRequest) XXX_Size() int {
	return xxx_messageInfo_FetchIntoObjectPoolRequest.Size(m)
}
func (m *FetchIntoObjectPoolRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_FetchIntoObjectPoolRequest.DiscardUnknown(m)
}

var xxx_messageInfo_FetchIntoObjectPoolRequest proto.InternalMessageInfo

func (m *FetchIntoObjectPoolRequest) GetOrigin() *Repository {
	if m != nil {
		return m.Origin
	}
	return nil
}

func (m *FetchIntoObjectPoolRequest) GetObjectPool() *ObjectPool {
	if m != nil {
		return m.ObjectPool
	}
	return nil
}

func (m *FetchIntoObjectPoolRequest) GetRepack() bool {
	if m != nil {
		return m.Repack
	}
	return false
}

type FetchIntoObjectPoolResponse struct {
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *FetchIntoObjectPoolResponse) Reset()         { *m = FetchIntoObjectPoolResponse{} }
func (m *FetchIntoObjectPoolResponse) String() string { return proto.CompactTextString(m) }
func (*FetchIntoObjectPoolResponse) ProtoMessage()    {}
func (*FetchIntoObjectPoolResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{13}
}

func (m *FetchIntoObjectPoolResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_FetchIntoObjectPoolResponse.Unmarshal(m, b)
}
func (m *FetchIntoObjectPoolResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_FetchIntoObjectPoolResponse.Marshal(b, m, deterministic)
}
func (m *FetchIntoObjectPoolResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_FetchIntoObjectPoolResponse.Merge(m, src)
}
func (m *FetchIntoObjectPoolResponse) XXX_Size() int {
	return xxx_messageInfo_FetchIntoObjectPoolResponse.Size(m)
}
func (m *FetchIntoObjectPoolResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_FetchIntoObjectPoolResponse.DiscardUnknown(m)
}

var xxx_messageInfo_FetchIntoObjectPoolResponse proto.InternalMessageInfo

type GetObjectPoolRequest struct {
	Repository           *Repository `protobuf:"bytes,1,opt,name=repository,proto3" json:"repository,omitempty"`
	XXX_NoUnkeyedLiteral struct{}    `json:"-"`
	XXX_unrecognized     []byte      `json:"-"`
	XXX_sizecache        int32       `json:"-"`
}

func (m *GetObjectPoolRequest) Reset()         { *m = GetObjectPoolRequest{} }
func (m *GetObjectPoolRequest) String() string { return proto.CompactTextString(m) }
func (*GetObjectPoolRequest) ProtoMessage()    {}
func (*GetObjectPoolRequest) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{14}
}

func (m *GetObjectPoolRequest) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_GetObjectPoolRequest.Unmarshal(m, b)
}
func (m *GetObjectPoolRequest) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_GetObjectPoolRequest.Marshal(b, m, deterministic)
}
func (m *GetObjectPoolRequest) XXX_Merge(src proto.Message) {
	xxx_messageInfo_GetObjectPoolRequest.Merge(m, src)
}
func (m *GetObjectPoolRequest) XXX_Size() int {
	return xxx_messageInfo_GetObjectPoolRequest.Size(m)
}
func (m *GetObjectPoolRequest) XXX_DiscardUnknown() {
	xxx_messageInfo_GetObjectPoolRequest.DiscardUnknown(m)
}

var xxx_messageInfo_GetObjectPoolRequest proto.InternalMessageInfo

func (m *GetObjectPoolRequest) GetRepository() *Repository {
	if m != nil {
		return m.Repository
	}
	return nil
}

type GetObjectPoolResponse struct {
	ObjectPool           *ObjectPool `protobuf:"bytes,1,opt,name=object_pool,json=objectPool,proto3" json:"object_pool,omitempty"`
	XXX_NoUnkeyedLiteral struct{}    `json:"-"`
	XXX_unrecognized     []byte      `json:"-"`
	XXX_sizecache        int32       `json:"-"`
}

func (m *GetObjectPoolResponse) Reset()         { *m = GetObjectPoolResponse{} }
func (m *GetObjectPoolResponse) String() string { return proto.CompactTextString(m) }
func (*GetObjectPoolResponse) ProtoMessage()    {}
func (*GetObjectPoolResponse) Descriptor() ([]byte, []int) {
	return fileDescriptor_949871727a345eae, []int{15}
}

func (m *GetObjectPoolResponse) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_GetObjectPoolResponse.Unmarshal(m, b)
}
func (m *GetObjectPoolResponse) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_GetObjectPoolResponse.Marshal(b, m, deterministic)
}
func (m *GetObjectPoolResponse) XXX_Merge(src proto.Message) {
	xxx_messageInfo_GetObjectPoolResponse.Merge(m, src)
}
func (m *GetObjectPoolResponse) XXX_Size() int {
	return xxx_messageInfo_GetObjectPoolResponse.Size(m)
}
func (m *GetObjectPoolResponse) XXX_DiscardUnknown() {
	xxx_messageInfo_GetObjectPoolResponse.DiscardUnknown(m)
}

var xxx_messageInfo_GetObjectPoolResponse proto.InternalMessageInfo

func (m *GetObjectPoolResponse) GetObjectPool() *ObjectPool {
	if m != nil {
		return m.ObjectPool
	}
	return nil
}

func init() {
	proto.RegisterType((*CreateObjectPoolRequest)(nil), "gitaly.CreateObjectPoolRequest")
	proto.RegisterType((*CreateObjectPoolResponse)(nil), "gitaly.CreateObjectPoolResponse")
	proto.RegisterType((*DeleteObjectPoolRequest)(nil), "gitaly.DeleteObjectPoolRequest")
	proto.RegisterType((*DeleteObjectPoolResponse)(nil), "gitaly.DeleteObjectPoolResponse")
	proto.RegisterType((*LinkRepositoryToObjectPoolRequest)(nil), "gitaly.LinkRepositoryToObjectPoolRequest")
	proto.RegisterType((*LinkRepositoryToObjectPoolResponse)(nil), "gitaly.LinkRepositoryToObjectPoolResponse")
	proto.RegisterType((*UnlinkRepositoryFromObjectPoolRequest)(nil), "gitaly.UnlinkRepositoryFromObjectPoolRequest")
	proto.RegisterType((*UnlinkRepositoryFromObjectPoolResponse)(nil), "gitaly.UnlinkRepositoryFromObjectPoolResponse")
	proto.RegisterType((*ReduplicateRepositoryRequest)(nil), "gitaly.ReduplicateRepositoryRequest")
	proto.RegisterType((*ReduplicateRepositoryResponse)(nil), "gitaly.ReduplicateRepositoryResponse")
	proto.RegisterType((*DisconnectGitAlternatesRequest)(nil), "gitaly.DisconnectGitAlternatesRequest")
	proto.RegisterType((*DisconnectGitAlternatesResponse)(nil), "gitaly.DisconnectGitAlternatesResponse")
	proto.RegisterType((*FetchIntoObjectPoolRequest)(nil), "gitaly.FetchIntoObjectPoolRequest")
	proto.RegisterType((*FetchIntoObjectPoolResponse)(nil), "gitaly.FetchIntoObjectPoolResponse")
	proto.RegisterType((*GetObjectPoolRequest)(nil), "gitaly.GetObjectPoolRequest")
	proto.RegisterType((*GetObjectPoolResponse)(nil), "gitaly.GetObjectPoolResponse")
}

func init() { proto.RegisterFile("objectpool.proto", fileDescriptor_949871727a345eae) }

var fileDescriptor_949871727a345eae = []byte{
	// 597 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0xac, 0x56, 0xd1, 0x4e, 0x13, 0x4d,
	0x18, 0xcd, 0xf4, 0x27, 0x1b, 0xf2, 0xf1, 0x9b, 0xe0, 0x28, 0xb6, 0x19, 0x29, 0x2d, 0x2b, 0x60,
	0x35, 0xd2, 0x6a, 0xe1, 0x42, 0x2f, 0xad, 0x04, 0x62, 0x42, 0x22, 0xa9, 0x35, 0x31, 0x24, 0xc6,
	0x6c, 0x97, 0xcf, 0x32, 0xb2, 0xec, 0xac, 0xb3, 0x03, 0x09, 0xf1, 0x5e, 0x1f, 0x41, 0x2e, 0xbc,
	0xc0, 0x97, 0xf1, 0x99, 0x8c, 0x57, 0xa6, 0xdd, 0xdd, 0xae, 0x9d, 0x76, 0xb6, 0x6b, 0xca, 0x5d,
	0x77, 0xf6, 0xcc, 0x39, 0xe7, 0x9b, 0x9d, 0x73, 0x52, 0x58, 0x14, 0xdd, 0x8f, 0xe8, 0xaa, 0x40,
	0x08, 0xaf, 0x1e, 0x48, 0xa1, 0x04, 0xb5, 0x7a, 0x5c, 0x39, 0xde, 0x05, 0x03, 0x8f, 0xfb, 0x2a,
	0x5a, 0x63, 0xff, 0x87, 0xc7, 0x8e, 0xc4, 0xa3, 0xe8, 0xc9, 0xfe, 0x42, 0xa0, 0xf8, 0x42, 0xa2,
	0xa3, 0xf0, 0xd5, 0x60, 0xf3, 0x81, 0x10, 0x5e, 0x1b, 0x3f, 0x9d, 0x61, 0xa8, 0xe8, 0x33, 0x58,
	0x88, 0x18, 0xdf, 0xf7, 0x29, 0x4b, 0xa4, 0x4a, 0x6a, 0x0b, 0x4d, 0x5a, 0x8f, 0x38, 0xeb, 0x29,
	0xbe, 0x35, 0x77, 0xf9, 0xf3, 0x11, 0x69, 0x83, 0x18, 0xae, 0xd0, 0xc7, 0x60, 0x09, 0xc9, 0x7b,
	0xdc, 0x2f, 0x15, 0x46, 0x77, 0xb5, 0x31, 0x10, 0x21, 0x57, 0x42, 0x5e, 0xb4, 0xe6, 0xae, 0xfa,
	0xbb, 0x62, 0x9c, 0xcd, 0xa0, 0x34, 0xee, 0x23, 0x0c, 0x84, 0x1f, 0xa2, 0xdd, 0x81, 0xe2, 0x0e,
	0x7a, 0x78, 0xbd, 0x1e, 0xfb, 0x8a, 0xe3, 0xac, 0xb1, 0xe2, 0x25, 0x81, 0xd5, 0x7d, 0xee, 0x9f,
	0xa4, 0x76, 0x3b, 0x62, 0x16, 0xf1, 0x2b, 0xfd, 0x80, 0x9e, 0x02, 0xc8, 0x21, 0x77, 0xd6, 0x21,
	0x45, 0xb6, 0x53, 0xac, 0xbd, 0x06, 0x76, 0x96, 0xb3, 0x78, 0x80, 0xef, 0x04, 0xd6, 0xdf, 0xf8,
	0xde, 0x08, 0x70, 0x57, 0x8a, 0xd3, 0xf1, 0x21, 0x46, 0x9d, 0x90, 0xfc, 0x4e, 0xf4, 0xf1, 0x0b,
	0xf9, 0xc7, 0xb7, 0x6b, 0xb0, 0x31, 0xcd, 0x5d, 0x3c, 0xc8, 0x5b, 0x58, 0x6e, 0xe3, 0xd1, 0x59,
	0xe0, 0x71, 0xd7, 0x51, 0x98, 0xc2, 0x67, 0xb6, 0x6f, 0x57, 0xa0, 0x6c, 0x60, 0x8e, 0xa5, 0x0f,
	0x61, 0x65, 0x87, 0x87, 0xae, 0xf0, 0x7d, 0x74, 0xd5, 0x1e, 0x57, 0xcf, 0x3d, 0x85, 0xd2, 0x77,
	0x14, 0x86, 0xb3, 0x8b, 0xaf, 0x42, 0xc5, 0xc8, 0x1d, 0xcb, 0xff, 0x20, 0xc0, 0x76, 0x51, 0xb9,
	0xc7, 0x2f, 0x7d, 0x35, 0xe1, 0xf2, 0xa5, 0x11, 0x23, 0xf9, 0x22, 0xf6, 0x0f, 0xdf, 0x6b, 0x2c,
	0xcf, 0x77, 0xc0, 0x92, 0x18, 0x38, 0xee, 0x49, 0xe9, 0xbf, 0x2a, 0xa9, 0xcd, 0xb7, 0xe3, 0x27,
	0xbb, 0x0c, 0x77, 0x27, 0x5a, 0x8c, 0x47, 0x38, 0x80, 0xdb, 0x7b, 0xa8, 0xae, 0xf1, 0xce, 0xd9,
	0xfb, 0xb0, 0xa4, 0x31, 0x46, 0x52, 0x74, 0x2b, 0x67, 0x16, 0xff, 0x1e, 0xab, 0xf9, 0xcb, 0x82,
	0x9b, 0xe9, 0xab, 0xd7, 0x28, 0xcf, 0xb9, 0x8b, 0xf4, 0x1d, 0x2c, 0xea, 0x55, 0x44, 0x2b, 0x09,
	0x93, 0xa1, 0x2c, 0x59, 0xd5, 0x0c, 0x88, 0x0f, 0xc3, 0xfa, 0xfd, 0xad, 0x56, 0x98, 0x27, 0x7d,
	0x7a, 0xbd, 0x77, 0x52, 0x7a, 0x43, 0xcf, 0xa5, 0xf4, 0xc6, 0xca, 0x4a, 0xe8, 0x3f, 0x03, 0x33,
	0xf7, 0x03, 0x7d, 0x90, 0xf0, 0x4c, 0x6d, 0x37, 0xf6, 0x30, 0x0f, 0x54, 0x13, 0xff, 0x4a, 0x60,
	0x25, 0x3b, 0xd8, 0x74, 0x33, 0xa1, 0xcd, 0x55, 0x4f, 0xac, 0x9e, 0x17, 0xae, 0x39, 0xf1, 0x60,
	0x69, 0x62, 0xba, 0xe9, 0x5a, 0x7a, 0xcf, 0xcc, 0xb5, 0xc2, 0xd6, 0xa7, 0xa0, 0x34, 0x35, 0x09,
	0x45, 0x43, 0x9c, 0xe9, 0xc6, 0xf0, 0xcb, 0x65, 0x76, 0x09, 0xbb, 0x3f, 0x15, 0xa7, 0x69, 0x7e,
	0x80, 0x5b, 0x13, 0xb2, 0x47, 0xed, 0x84, 0xc7, 0xdc, 0x1d, 0xec, 0x5e, 0x26, 0x46, 0xd3, 0xe9,
	0xc0, 0x8d, 0x91, 0xc8, 0xd1, 0xe5, 0x64, 0xf7, 0xa4, 0x6c, 0xb3, 0xb2, 0xe1, 0xed, 0x08, 0x6b,
	0xa1, 0xb5, 0x7d, 0xd8, 0xec, 0x71, 0xe5, 0x39, 0xdd, 0xba, 0x2b, 0x4e, 0x1b, 0xd1, 0xcf, 0x4d,
	0x21, 0x7b, 0x8d, 0x68, 0x77, 0xe3, 0xfc, 0xc9, 0x76, 0x63, 0xf0, 0x17, 0xa5, 0xd1, 0x13, 0xf1,
	0x5a, 0xd0, 0xed, 0x5a, 0x83, 0xa5, 0xad, 0x3f, 0x01, 0x00, 0x00, 0xff, 0xff, 0x71, 0xf1, 0x8a,
	0xca, 0xeb, 0x08, 0x00, 0x00,
}

// Reference imports to suppress errors if they are not otherwise used.
var _ context.Context
var _ grpc.ClientConn

// This is a compile-time assertion to ensure that this generated file
// is compatible with the grpc package it is being compiled against.
const _ = grpc.SupportPackageIsVersion4

// ObjectPoolServiceClient is the client API for ObjectPoolService service.
//
// For semantics around ctx use and closing/ending streaming RPCs, please refer to https://godoc.org/google.golang.org/grpc#ClientConn.NewStream.
type ObjectPoolServiceClient interface {
	CreateObjectPool(ctx context.Context, in *CreateObjectPoolRequest, opts ...grpc.CallOption) (*CreateObjectPoolResponse, error)
	DeleteObjectPool(ctx context.Context, in *DeleteObjectPoolRequest, opts ...grpc.CallOption) (*DeleteObjectPoolResponse, error)
	// Repositories are assumed to be stored on the same disk
	LinkRepositoryToObjectPool(ctx context.Context, in *LinkRepositoryToObjectPoolRequest, opts ...grpc.CallOption) (*LinkRepositoryToObjectPoolResponse, error)
	UnlinkRepositoryFromObjectPool(ctx context.Context, in *UnlinkRepositoryFromObjectPoolRequest, opts ...grpc.CallOption) (*UnlinkRepositoryFromObjectPoolResponse, error)
	ReduplicateRepository(ctx context.Context, in *ReduplicateRepositoryRequest, opts ...grpc.CallOption) (*ReduplicateRepositoryResponse, error)
	DisconnectGitAlternates(ctx context.Context, in *DisconnectGitAlternatesRequest, opts ...grpc.CallOption) (*DisconnectGitAlternatesResponse, error)
	FetchIntoObjectPool(ctx context.Context, in *FetchIntoObjectPoolRequest, opts ...grpc.CallOption) (*FetchIntoObjectPoolResponse, error)
	GetObjectPool(ctx context.Context, in *GetObjectPoolRequest, opts ...grpc.CallOption) (*GetObjectPoolResponse, error)
}

type objectPoolServiceClient struct {
	cc *grpc.ClientConn
}

func NewObjectPoolServiceClient(cc *grpc.ClientConn) ObjectPoolServiceClient {
	return &objectPoolServiceClient{cc}
}

func (c *objectPoolServiceClient) CreateObjectPool(ctx context.Context, in *CreateObjectPoolRequest, opts ...grpc.CallOption) (*CreateObjectPoolResponse, error) {
	out := new(CreateObjectPoolResponse)
	err := c.cc.Invoke(ctx, "/gitaly.ObjectPoolService/CreateObjectPool", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *objectPoolServiceClient) DeleteObjectPool(ctx context.Context, in *DeleteObjectPoolRequest, opts ...grpc.CallOption) (*DeleteObjectPoolResponse, error) {
	out := new(DeleteObjectPoolResponse)
	err := c.cc.Invoke(ctx, "/gitaly.ObjectPoolService/DeleteObjectPool", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *objectPoolServiceClient) LinkRepositoryToObjectPool(ctx context.Context, in *LinkRepositoryToObjectPoolRequest, opts ...grpc.CallOption) (*LinkRepositoryToObjectPoolResponse, error) {
	out := new(LinkRepositoryToObjectPoolResponse)
	err := c.cc.Invoke(ctx, "/gitaly.ObjectPoolService/LinkRepositoryToObjectPool", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *objectPoolServiceClient) UnlinkRepositoryFromObjectPool(ctx context.Context, in *UnlinkRepositoryFromObjectPoolRequest, opts ...grpc.CallOption) (*UnlinkRepositoryFromObjectPoolResponse, error) {
	out := new(UnlinkRepositoryFromObjectPoolResponse)
	err := c.cc.Invoke(ctx, "/gitaly.ObjectPoolService/UnlinkRepositoryFromObjectPool", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *objectPoolServiceClient) ReduplicateRepository(ctx context.Context, in *ReduplicateRepositoryRequest, opts ...grpc.CallOption) (*ReduplicateRepositoryResponse, error) {
	out := new(ReduplicateRepositoryResponse)
	err := c.cc.Invoke(ctx, "/gitaly.ObjectPoolService/ReduplicateRepository", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *objectPoolServiceClient) DisconnectGitAlternates(ctx context.Context, in *DisconnectGitAlternatesRequest, opts ...grpc.CallOption) (*DisconnectGitAlternatesResponse, error) {
	out := new(DisconnectGitAlternatesResponse)
	err := c.cc.Invoke(ctx, "/gitaly.ObjectPoolService/DisconnectGitAlternates", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *objectPoolServiceClient) FetchIntoObjectPool(ctx context.Context, in *FetchIntoObjectPoolRequest, opts ...grpc.CallOption) (*FetchIntoObjectPoolResponse, error) {
	out := new(FetchIntoObjectPoolResponse)
	err := c.cc.Invoke(ctx, "/gitaly.ObjectPoolService/FetchIntoObjectPool", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

func (c *objectPoolServiceClient) GetObjectPool(ctx context.Context, in *GetObjectPoolRequest, opts ...grpc.CallOption) (*GetObjectPoolResponse, error) {
	out := new(GetObjectPoolResponse)
	err := c.cc.Invoke(ctx, "/gitaly.ObjectPoolService/GetObjectPool", in, out, opts...)
	if err != nil {
		return nil, err
	}
	return out, nil
}

// ObjectPoolServiceServer is the server API for ObjectPoolService service.
type ObjectPoolServiceServer interface {
	CreateObjectPool(context.Context, *CreateObjectPoolRequest) (*CreateObjectPoolResponse, error)
	DeleteObjectPool(context.Context, *DeleteObjectPoolRequest) (*DeleteObjectPoolResponse, error)
	// Repositories are assumed to be stored on the same disk
	LinkRepositoryToObjectPool(context.Context, *LinkRepositoryToObjectPoolRequest) (*LinkRepositoryToObjectPoolResponse, error)
	UnlinkRepositoryFromObjectPool(context.Context, *UnlinkRepositoryFromObjectPoolRequest) (*UnlinkRepositoryFromObjectPoolResponse, error)
	ReduplicateRepository(context.Context, *ReduplicateRepositoryRequest) (*ReduplicateRepositoryResponse, error)
	DisconnectGitAlternates(context.Context, *DisconnectGitAlternatesRequest) (*DisconnectGitAlternatesResponse, error)
	FetchIntoObjectPool(context.Context, *FetchIntoObjectPoolRequest) (*FetchIntoObjectPoolResponse, error)
	GetObjectPool(context.Context, *GetObjectPoolRequest) (*GetObjectPoolResponse, error)
}

// UnimplementedObjectPoolServiceServer can be embedded to have forward compatible implementations.
type UnimplementedObjectPoolServiceServer struct {
}

func (*UnimplementedObjectPoolServiceServer) CreateObjectPool(ctx context.Context, req *CreateObjectPoolRequest) (*CreateObjectPoolResponse, error) {
	return nil, status.Errorf(codes.Unimplemented, "method CreateObjectPool not implemented")
}
func (*UnimplementedObjectPoolServiceServer) DeleteObjectPool(ctx context.Context, req *DeleteObjectPoolRequest) (*DeleteObjectPoolResponse, error) {
	return nil, status.Errorf(codes.Unimplemented, "method DeleteObjectPool not implemented")
}
func (*UnimplementedObjectPoolServiceServer) LinkRepositoryToObjectPool(ctx context.Context, req *LinkRepositoryToObjectPoolRequest) (*LinkRepositoryToObjectPoolResponse, error) {
	return nil, status.Errorf(codes.Unimplemented, "method LinkRepositoryToObjectPool not implemented")
}
func (*UnimplementedObjectPoolServiceServer) UnlinkRepositoryFromObjectPool(ctx context.Context, req *UnlinkRepositoryFromObjectPoolRequest) (*UnlinkRepositoryFromObjectPoolResponse, error) {
	return nil, status.Errorf(codes.Unimplemented, "method UnlinkRepositoryFromObjectPool not implemented")
}
func (*UnimplementedObjectPoolServiceServer) ReduplicateRepository(ctx context.Context, req *ReduplicateRepositoryRequest) (*ReduplicateRepositoryResponse, error) {
	return nil, status.Errorf(codes.Unimplemented, "method ReduplicateRepository not implemented")
}
func (*UnimplementedObjectPoolServiceServer) DisconnectGitAlternates(ctx context.Context, req *DisconnectGitAlternatesRequest) (*DisconnectGitAlternatesResponse, error) {
	return nil, status.Errorf(codes.Unimplemented, "method DisconnectGitAlternates not implemented")
}
func (*UnimplementedObjectPoolServiceServer) FetchIntoObjectPool(ctx context.Context, req *FetchIntoObjectPoolRequest) (*FetchIntoObjectPoolResponse, error) {
	return nil, status.Errorf(codes.Unimplemented, "method FetchIntoObjectPool not implemented")
}
func (*UnimplementedObjectPoolServiceServer) GetObjectPool(ctx context.Context, req *GetObjectPoolRequest) (*GetObjectPoolResponse, error) {
	return nil, status.Errorf(codes.Unimplemented, "method GetObjectPool not implemented")
}

func RegisterObjectPoolServiceServer(s *grpc.Server, srv ObjectPoolServiceServer) {
	s.RegisterService(&_ObjectPoolService_serviceDesc, srv)
}

func _ObjectPoolService_CreateObjectPool_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(CreateObjectPoolRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ObjectPoolServiceServer).CreateObjectPool(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/gitaly.ObjectPoolService/CreateObjectPool",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ObjectPoolServiceServer).CreateObjectPool(ctx, req.(*CreateObjectPoolRequest))
	}
	return interceptor(ctx, in, info, handler)
}

func _ObjectPoolService_DeleteObjectPool_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(DeleteObjectPoolRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ObjectPoolServiceServer).DeleteObjectPool(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/gitaly.ObjectPoolService/DeleteObjectPool",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ObjectPoolServiceServer).DeleteObjectPool(ctx, req.(*DeleteObjectPoolRequest))
	}
	return interceptor(ctx, in, info, handler)
}

func _ObjectPoolService_LinkRepositoryToObjectPool_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(LinkRepositoryToObjectPoolRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ObjectPoolServiceServer).LinkRepositoryToObjectPool(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/gitaly.ObjectPoolService/LinkRepositoryToObjectPool",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ObjectPoolServiceServer).LinkRepositoryToObjectPool(ctx, req.(*LinkRepositoryToObjectPoolRequest))
	}
	return interceptor(ctx, in, info, handler)
}

func _ObjectPoolService_UnlinkRepositoryFromObjectPool_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(UnlinkRepositoryFromObjectPoolRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ObjectPoolServiceServer).UnlinkRepositoryFromObjectPool(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/gitaly.ObjectPoolService/UnlinkRepositoryFromObjectPool",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ObjectPoolServiceServer).UnlinkRepositoryFromObjectPool(ctx, req.(*UnlinkRepositoryFromObjectPoolRequest))
	}
	return interceptor(ctx, in, info, handler)
}

func _ObjectPoolService_ReduplicateRepository_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(ReduplicateRepositoryRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ObjectPoolServiceServer).ReduplicateRepository(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/gitaly.ObjectPoolService/ReduplicateRepository",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ObjectPoolServiceServer).ReduplicateRepository(ctx, req.(*ReduplicateRepositoryRequest))
	}
	return interceptor(ctx, in, info, handler)
}

func _ObjectPoolService_DisconnectGitAlternates_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(DisconnectGitAlternatesRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ObjectPoolServiceServer).DisconnectGitAlternates(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/gitaly.ObjectPoolService/DisconnectGitAlternates",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ObjectPoolServiceServer).DisconnectGitAlternates(ctx, req.(*DisconnectGitAlternatesRequest))
	}
	return interceptor(ctx, in, info, handler)
}

func _ObjectPoolService_FetchIntoObjectPool_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(FetchIntoObjectPoolRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ObjectPoolServiceServer).FetchIntoObjectPool(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/gitaly.ObjectPoolService/FetchIntoObjectPool",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ObjectPoolServiceServer).FetchIntoObjectPool(ctx, req.(*FetchIntoObjectPoolRequest))
	}
	return interceptor(ctx, in, info, handler)
}

func _ObjectPoolService_GetObjectPool_Handler(srv interface{}, ctx context.Context, dec func(interface{}) error, interceptor grpc.UnaryServerInterceptor) (interface{}, error) {
	in := new(GetObjectPoolRequest)
	if err := dec(in); err != nil {
		return nil, err
	}
	if interceptor == nil {
		return srv.(ObjectPoolServiceServer).GetObjectPool(ctx, in)
	}
	info := &grpc.UnaryServerInfo{
		Server:     srv,
		FullMethod: "/gitaly.ObjectPoolService/GetObjectPool",
	}
	handler := func(ctx context.Context, req interface{}) (interface{}, error) {
		return srv.(ObjectPoolServiceServer).GetObjectPool(ctx, req.(*GetObjectPoolRequest))
	}
	return interceptor(ctx, in, info, handler)
}

var _ObjectPoolService_serviceDesc = grpc.ServiceDesc{
	ServiceName: "gitaly.ObjectPoolService",
	HandlerType: (*ObjectPoolServiceServer)(nil),
	Methods: []grpc.MethodDesc{
		{
			MethodName: "CreateObjectPool",
			Handler:    _ObjectPoolService_CreateObjectPool_Handler,
		},
		{
			MethodName: "DeleteObjectPool",
			Handler:    _ObjectPoolService_DeleteObjectPool_Handler,
		},
		{
			MethodName: "LinkRepositoryToObjectPool",
			Handler:    _ObjectPoolService_LinkRepositoryToObjectPool_Handler,
		},
		{
			MethodName: "UnlinkRepositoryFromObjectPool",
			Handler:    _ObjectPoolService_UnlinkRepositoryFromObjectPool_Handler,
		},
		{
			MethodName: "ReduplicateRepository",
			Handler:    _ObjectPoolService_ReduplicateRepository_Handler,
		},
		{
			MethodName: "DisconnectGitAlternates",
			Handler:    _ObjectPoolService_DisconnectGitAlternates_Handler,
		},
		{
			MethodName: "FetchIntoObjectPool",
			Handler:    _ObjectPoolService_FetchIntoObjectPool_Handler,
		},
		{
			MethodName: "GetObjectPool",
			Handler:    _ObjectPoolService_GetObjectPool_Handler,
		},
	},
	Streams:  []grpc.StreamDesc{},
	Metadata: "objectpool.proto",
}
