# Generated by the protocol buffer compiler.  DO NOT EDIT!
# Source: cleanup.proto for package 'gitaly'

require 'grpc'
require 'cleanup_pb'

module Gitaly
  module CleanupService
    class Service

      include GRPC::GenericService

      self.marshal_class_method = :encode
      self.unmarshal_class_method = :decode
      self.service_name = 'gitaly.CleanupService'

      rpc :ApplyBfgObjectMapStream, stream(Gitaly::ApplyBfgObjectMapStreamRequest), stream(Gitaly::ApplyBfgObjectMapStreamResponse)
    end

    Stub = Service.rpc_stub_class
  end
end
