// Code generated by protoc-gen-go. DO NOT EDIT.
// source: shared.proto

package gitalypb

import (
	fmt "fmt"
	proto "github.com/golang/protobuf/proto"
	timestamp "github.com/golang/protobuf/ptypes/timestamp"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

type ObjectType int32

const (
	ObjectType_UNKNOWN ObjectType = 0
	ObjectType_COMMIT  ObjectType = 1
	ObjectType_BLOB    ObjectType = 2
	ObjectType_TREE    ObjectType = 3
	ObjectType_TAG     ObjectType = 4
)

var ObjectType_name = map[int32]string{
	0: "UNKNOWN",
	1: "COMMIT",
	2: "BLOB",
	3: "TREE",
	4: "TAG",
}

var ObjectType_value = map[string]int32{
	"UNKNOWN": 0,
	"COMMIT":  1,
	"BLOB":    2,
	"TREE":    3,
	"TAG":     4,
}

func (x ObjectType) String() string {
	return proto.EnumName(ObjectType_name, int32(x))
}

func (ObjectType) EnumDescriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{0}
}

type SignatureType int32

const (
	SignatureType_NONE SignatureType = 0
	SignatureType_PGP  SignatureType = 1
	SignatureType_X509 SignatureType = 2
)

var SignatureType_name = map[int32]string{
	0: "NONE",
	1: "PGP",
	2: "X509",
}

var SignatureType_value = map[string]int32{
	"NONE": 0,
	"PGP":  1,
	"X509": 2,
}

func (x SignatureType) String() string {
	return proto.EnumName(SignatureType_name, int32(x))
}

func (SignatureType) EnumDescriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{1}
}

type Repository struct {
	StorageName  string `protobuf:"bytes,2,opt,name=storage_name,json=storageName,proto3" json:"storage_name,omitempty"`
	RelativePath string `protobuf:"bytes,3,opt,name=relative_path,json=relativePath,proto3" json:"relative_path,omitempty"`
	// Sets the GIT_OBJECT_DIRECTORY envvar on git commands to the value of this field.
	// It influences the object storage directory the SHA1 directories are created underneath.
	GitObjectDirectory string `protobuf:"bytes,4,opt,name=git_object_directory,json=gitObjectDirectory,proto3" json:"git_object_directory,omitempty"`
	// Sets the GIT_ALTERNATE_OBJECT_DIRECTORIES envvar on git commands to the values of this field.
	// It influences the list of Git object directories which can be used to search for Git objects.
	GitAlternateObjectDirectories []string `protobuf:"bytes,5,rep,name=git_alternate_object_directories,json=gitAlternateObjectDirectories,proto3" json:"git_alternate_object_directories,omitempty"`
	// Used in callbacks to GitLab so that it knows what repository the event is
	// associated with. May be left empty on RPC's that do not perform callbacks.
	// During project creation, `gl_repository` may not be known.
	GlRepository string `protobuf:"bytes,6,opt,name=gl_repository,json=glRepository,proto3" json:"gl_repository,omitempty"`
	// The human-readable GitLab project path (e.g. gitlab-org/gitlab-ce).
	// When hashed storage is use, this associates a project path with its
	// path on disk. The name can change over time (e.g. when a project is
	// renamed). This is primarily used for logging/debugging at the
	// moment.
	GlProjectPath        string   `protobuf:"bytes,8,opt,name=gl_project_path,json=glProjectPath,proto3" json:"gl_project_path,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *Repository) Reset()         { *m = Repository{} }
func (m *Repository) String() string { return proto.CompactTextString(m) }
func (*Repository) ProtoMessage()    {}
func (*Repository) Descriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{0}
}

func (m *Repository) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_Repository.Unmarshal(m, b)
}
func (m *Repository) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_Repository.Marshal(b, m, deterministic)
}
func (m *Repository) XXX_Merge(src proto.Message) {
	xxx_messageInfo_Repository.Merge(m, src)
}
func (m *Repository) XXX_Size() int {
	return xxx_messageInfo_Repository.Size(m)
}
func (m *Repository) XXX_DiscardUnknown() {
	xxx_messageInfo_Repository.DiscardUnknown(m)
}

var xxx_messageInfo_Repository proto.InternalMessageInfo

func (m *Repository) GetStorageName() string {
	if m != nil {
		return m.StorageName
	}
	return ""
}

func (m *Repository) GetRelativePath() string {
	if m != nil {
		return m.RelativePath
	}
	return ""
}

func (m *Repository) GetGitObjectDirectory() string {
	if m != nil {
		return m.GitObjectDirectory
	}
	return ""
}

func (m *Repository) GetGitAlternateObjectDirectories() []string {
	if m != nil {
		return m.GitAlternateObjectDirectories
	}
	return nil
}

func (m *Repository) GetGlRepository() string {
	if m != nil {
		return m.GlRepository
	}
	return ""
}

func (m *Repository) GetGlProjectPath() string {
	if m != nil {
		return m.GlProjectPath
	}
	return ""
}

// A single Git trailer (https://git-scm.com/docs/git-interpret-trailers)
// key-value pair.
type CommitTrailer struct {
	// The key of the trailer, such as `Signed-off-by`.
	Key []byte `protobuf:"bytes,1,opt,name=key,proto3" json:"key,omitempty"`
	// The value of the trailer, such as `Alice <alice@gmail.com>`.
	Value                []byte   `protobuf:"bytes,2,opt,name=value,proto3" json:"value,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *CommitTrailer) Reset()         { *m = CommitTrailer{} }
func (m *CommitTrailer) String() string { return proto.CompactTextString(m) }
func (*CommitTrailer) ProtoMessage()    {}
func (*CommitTrailer) Descriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{1}
}

func (m *CommitTrailer) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_CommitTrailer.Unmarshal(m, b)
}
func (m *CommitTrailer) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_CommitTrailer.Marshal(b, m, deterministic)
}
func (m *CommitTrailer) XXX_Merge(src proto.Message) {
	xxx_messageInfo_CommitTrailer.Merge(m, src)
}
func (m *CommitTrailer) XXX_Size() int {
	return xxx_messageInfo_CommitTrailer.Size(m)
}
func (m *CommitTrailer) XXX_DiscardUnknown() {
	xxx_messageInfo_CommitTrailer.DiscardUnknown(m)
}

var xxx_messageInfo_CommitTrailer proto.InternalMessageInfo

func (m *CommitTrailer) GetKey() []byte {
	if m != nil {
		return m.Key
	}
	return nil
}

func (m *CommitTrailer) GetValue() []byte {
	if m != nil {
		return m.Value
	}
	return nil
}

// Corresponds to Gitlab::Git::Commit
type GitCommit struct {
	Id        string        `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	Subject   []byte        `protobuf:"bytes,2,opt,name=subject,proto3" json:"subject,omitempty"`
	Body      []byte        `protobuf:"bytes,3,opt,name=body,proto3" json:"body,omitempty"`
	Author    *CommitAuthor `protobuf:"bytes,4,opt,name=author,proto3" json:"author,omitempty"`
	Committer *CommitAuthor `protobuf:"bytes,5,opt,name=committer,proto3" json:"committer,omitempty"`
	ParentIds []string      `protobuf:"bytes,6,rep,name=parent_ids,json=parentIds,proto3" json:"parent_ids,omitempty"`
	// If body exceeds a certain threshold, it will be nullified,
	// but its size will be set in body_size so we can know if
	// a commit had a body in the first place.
	BodySize      int64         `protobuf:"varint,7,opt,name=body_size,json=bodySize,proto3" json:"body_size,omitempty"`
	SignatureType SignatureType `protobuf:"varint,8,opt,name=signature_type,json=signatureType,proto3,enum=gitaly.SignatureType" json:"signature_type,omitempty"`
	// The tree ID will always be filled, even if the tree is empty. In that case
	// the value will be `4b825dc642cb6eb9a060e54bf8d69288fbee4904`.
	// That value is equivalent to `git hash-object -t tree /dev/null`
	TreeId string `protobuf:"bytes,9,opt,name=tree_id,json=treeId,proto3" json:"tree_id,omitempty"`
	// The list of Git trailers (https://git-scm.com/docs/git-interpret-trailers)
	// found in this commit's message. The number of trailers and their key/value
	// sizes are limited. If a trailer exceeds these size limits, it and any
	// trailers that follow it are not included.
	Trailers             []*CommitTrailer `protobuf:"bytes,10,rep,name=trailers,proto3" json:"trailers,omitempty"`
	XXX_NoUnkeyedLiteral struct{}         `json:"-"`
	XXX_unrecognized     []byte           `json:"-"`
	XXX_sizecache        int32            `json:"-"`
}

func (m *GitCommit) Reset()         { *m = GitCommit{} }
func (m *GitCommit) String() string { return proto.CompactTextString(m) }
func (*GitCommit) ProtoMessage()    {}
func (*GitCommit) Descriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{2}
}

func (m *GitCommit) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_GitCommit.Unmarshal(m, b)
}
func (m *GitCommit) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_GitCommit.Marshal(b, m, deterministic)
}
func (m *GitCommit) XXX_Merge(src proto.Message) {
	xxx_messageInfo_GitCommit.Merge(m, src)
}
func (m *GitCommit) XXX_Size() int {
	return xxx_messageInfo_GitCommit.Size(m)
}
func (m *GitCommit) XXX_DiscardUnknown() {
	xxx_messageInfo_GitCommit.DiscardUnknown(m)
}

var xxx_messageInfo_GitCommit proto.InternalMessageInfo

func (m *GitCommit) GetId() string {
	if m != nil {
		return m.Id
	}
	return ""
}

func (m *GitCommit) GetSubject() []byte {
	if m != nil {
		return m.Subject
	}
	return nil
}

func (m *GitCommit) GetBody() []byte {
	if m != nil {
		return m.Body
	}
	return nil
}

func (m *GitCommit) GetAuthor() *CommitAuthor {
	if m != nil {
		return m.Author
	}
	return nil
}

func (m *GitCommit) GetCommitter() *CommitAuthor {
	if m != nil {
		return m.Committer
	}
	return nil
}

func (m *GitCommit) GetParentIds() []string {
	if m != nil {
		return m.ParentIds
	}
	return nil
}

func (m *GitCommit) GetBodySize() int64 {
	if m != nil {
		return m.BodySize
	}
	return 0
}

func (m *GitCommit) GetSignatureType() SignatureType {
	if m != nil {
		return m.SignatureType
	}
	return SignatureType_NONE
}

func (m *GitCommit) GetTreeId() string {
	if m != nil {
		return m.TreeId
	}
	return ""
}

func (m *GitCommit) GetTrailers() []*CommitTrailer {
	if m != nil {
		return m.Trailers
	}
	return nil
}

type CommitAuthor struct {
	Name                 []byte               `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Email                []byte               `protobuf:"bytes,2,opt,name=email,proto3" json:"email,omitempty"`
	Date                 *timestamp.Timestamp `protobuf:"bytes,3,opt,name=date,proto3" json:"date,omitempty"`
	Timezone             []byte               `protobuf:"bytes,4,opt,name=timezone,proto3" json:"timezone,omitempty"`
	XXX_NoUnkeyedLiteral struct{}             `json:"-"`
	XXX_unrecognized     []byte               `json:"-"`
	XXX_sizecache        int32                `json:"-"`
}

func (m *CommitAuthor) Reset()         { *m = CommitAuthor{} }
func (m *CommitAuthor) String() string { return proto.CompactTextString(m) }
func (*CommitAuthor) ProtoMessage()    {}
func (*CommitAuthor) Descriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{3}
}

func (m *CommitAuthor) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_CommitAuthor.Unmarshal(m, b)
}
func (m *CommitAuthor) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_CommitAuthor.Marshal(b, m, deterministic)
}
func (m *CommitAuthor) XXX_Merge(src proto.Message) {
	xxx_messageInfo_CommitAuthor.Merge(m, src)
}
func (m *CommitAuthor) XXX_Size() int {
	return xxx_messageInfo_CommitAuthor.Size(m)
}
func (m *CommitAuthor) XXX_DiscardUnknown() {
	xxx_messageInfo_CommitAuthor.DiscardUnknown(m)
}

var xxx_messageInfo_CommitAuthor proto.InternalMessageInfo

func (m *CommitAuthor) GetName() []byte {
	if m != nil {
		return m.Name
	}
	return nil
}

func (m *CommitAuthor) GetEmail() []byte {
	if m != nil {
		return m.Email
	}
	return nil
}

func (m *CommitAuthor) GetDate() *timestamp.Timestamp {
	if m != nil {
		return m.Date
	}
	return nil
}

func (m *CommitAuthor) GetTimezone() []byte {
	if m != nil {
		return m.Timezone
	}
	return nil
}

type ExitStatus struct {
	Value                int32    `protobuf:"varint,1,opt,name=value,proto3" json:"value,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *ExitStatus) Reset()         { *m = ExitStatus{} }
func (m *ExitStatus) String() string { return proto.CompactTextString(m) }
func (*ExitStatus) ProtoMessage()    {}
func (*ExitStatus) Descriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{4}
}

func (m *ExitStatus) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ExitStatus.Unmarshal(m, b)
}
func (m *ExitStatus) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ExitStatus.Marshal(b, m, deterministic)
}
func (m *ExitStatus) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ExitStatus.Merge(m, src)
}
func (m *ExitStatus) XXX_Size() int {
	return xxx_messageInfo_ExitStatus.Size(m)
}
func (m *ExitStatus) XXX_DiscardUnknown() {
	xxx_messageInfo_ExitStatus.DiscardUnknown(m)
}

var xxx_messageInfo_ExitStatus proto.InternalMessageInfo

func (m *ExitStatus) GetValue() int32 {
	if m != nil {
		return m.Value
	}
	return 0
}

// Corresponds to Gitlab::Git::Branch
type Branch struct {
	Name                 []byte     `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	TargetCommit         *GitCommit `protobuf:"bytes,2,opt,name=target_commit,json=targetCommit,proto3" json:"target_commit,omitempty"`
	XXX_NoUnkeyedLiteral struct{}   `json:"-"`
	XXX_unrecognized     []byte     `json:"-"`
	XXX_sizecache        int32      `json:"-"`
}

func (m *Branch) Reset()         { *m = Branch{} }
func (m *Branch) String() string { return proto.CompactTextString(m) }
func (*Branch) ProtoMessage()    {}
func (*Branch) Descriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{5}
}

func (m *Branch) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_Branch.Unmarshal(m, b)
}
func (m *Branch) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_Branch.Marshal(b, m, deterministic)
}
func (m *Branch) XXX_Merge(src proto.Message) {
	xxx_messageInfo_Branch.Merge(m, src)
}
func (m *Branch) XXX_Size() int {
	return xxx_messageInfo_Branch.Size(m)
}
func (m *Branch) XXX_DiscardUnknown() {
	xxx_messageInfo_Branch.DiscardUnknown(m)
}

var xxx_messageInfo_Branch proto.InternalMessageInfo

func (m *Branch) GetName() []byte {
	if m != nil {
		return m.Name
	}
	return nil
}

func (m *Branch) GetTargetCommit() *GitCommit {
	if m != nil {
		return m.TargetCommit
	}
	return nil
}

type Tag struct {
	Name         []byte     `protobuf:"bytes,1,opt,name=name,proto3" json:"name,omitempty"`
	Id           string     `protobuf:"bytes,2,opt,name=id,proto3" json:"id,omitempty"`
	TargetCommit *GitCommit `protobuf:"bytes,3,opt,name=target_commit,json=targetCommit,proto3" json:"target_commit,omitempty"`
	// If message exceeds a certain threshold, it will be nullified,
	// but its size will be set in message_size so we can know if
	// a tag had a message in the first place.
	Message              []byte        `protobuf:"bytes,4,opt,name=message,proto3" json:"message,omitempty"`
	MessageSize          int64         `protobuf:"varint,5,opt,name=message_size,json=messageSize,proto3" json:"message_size,omitempty"`
	Tagger               *CommitAuthor `protobuf:"bytes,6,opt,name=tagger,proto3" json:"tagger,omitempty"`
	SignatureType        SignatureType `protobuf:"varint,7,opt,name=signature_type,json=signatureType,proto3,enum=gitaly.SignatureType" json:"signature_type,omitempty"`
	XXX_NoUnkeyedLiteral struct{}      `json:"-"`
	XXX_unrecognized     []byte        `json:"-"`
	XXX_sizecache        int32         `json:"-"`
}

func (m *Tag) Reset()         { *m = Tag{} }
func (m *Tag) String() string { return proto.CompactTextString(m) }
func (*Tag) ProtoMessage()    {}
func (*Tag) Descriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{6}
}

func (m *Tag) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_Tag.Unmarshal(m, b)
}
func (m *Tag) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_Tag.Marshal(b, m, deterministic)
}
func (m *Tag) XXX_Merge(src proto.Message) {
	xxx_messageInfo_Tag.Merge(m, src)
}
func (m *Tag) XXX_Size() int {
	return xxx_messageInfo_Tag.Size(m)
}
func (m *Tag) XXX_DiscardUnknown() {
	xxx_messageInfo_Tag.DiscardUnknown(m)
}

var xxx_messageInfo_Tag proto.InternalMessageInfo

func (m *Tag) GetName() []byte {
	if m != nil {
		return m.Name
	}
	return nil
}

func (m *Tag) GetId() string {
	if m != nil {
		return m.Id
	}
	return ""
}

func (m *Tag) GetTargetCommit() *GitCommit {
	if m != nil {
		return m.TargetCommit
	}
	return nil
}

func (m *Tag) GetMessage() []byte {
	if m != nil {
		return m.Message
	}
	return nil
}

func (m *Tag) GetMessageSize() int64 {
	if m != nil {
		return m.MessageSize
	}
	return 0
}

func (m *Tag) GetTagger() *CommitAuthor {
	if m != nil {
		return m.Tagger
	}
	return nil
}

func (m *Tag) GetSignatureType() SignatureType {
	if m != nil {
		return m.SignatureType
	}
	return SignatureType_NONE
}

type User struct {
	GlId                 string   `protobuf:"bytes,1,opt,name=gl_id,json=glId,proto3" json:"gl_id,omitempty"`
	Name                 []byte   `protobuf:"bytes,2,opt,name=name,proto3" json:"name,omitempty"`
	Email                []byte   `protobuf:"bytes,3,opt,name=email,proto3" json:"email,omitempty"`
	GlUsername           string   `protobuf:"bytes,4,opt,name=gl_username,json=glUsername,proto3" json:"gl_username,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *User) Reset()         { *m = User{} }
func (m *User) String() string { return proto.CompactTextString(m) }
func (*User) ProtoMessage()    {}
func (*User) Descriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{7}
}

func (m *User) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_User.Unmarshal(m, b)
}
func (m *User) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_User.Marshal(b, m, deterministic)
}
func (m *User) XXX_Merge(src proto.Message) {
	xxx_messageInfo_User.Merge(m, src)
}
func (m *User) XXX_Size() int {
	return xxx_messageInfo_User.Size(m)
}
func (m *User) XXX_DiscardUnknown() {
	xxx_messageInfo_User.DiscardUnknown(m)
}

var xxx_messageInfo_User proto.InternalMessageInfo

func (m *User) GetGlId() string {
	if m != nil {
		return m.GlId
	}
	return ""
}

func (m *User) GetName() []byte {
	if m != nil {
		return m.Name
	}
	return nil
}

func (m *User) GetEmail() []byte {
	if m != nil {
		return m.Email
	}
	return nil
}

func (m *User) GetGlUsername() string {
	if m != nil {
		return m.GlUsername
	}
	return ""
}

type ObjectPool struct {
	Repository           *Repository `protobuf:"bytes,1,opt,name=repository,proto3" json:"repository,omitempty"`
	XXX_NoUnkeyedLiteral struct{}    `json:"-"`
	XXX_unrecognized     []byte      `json:"-"`
	XXX_sizecache        int32       `json:"-"`
}

func (m *ObjectPool) Reset()         { *m = ObjectPool{} }
func (m *ObjectPool) String() string { return proto.CompactTextString(m) }
func (*ObjectPool) ProtoMessage()    {}
func (*ObjectPool) Descriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{8}
}

func (m *ObjectPool) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ObjectPool.Unmarshal(m, b)
}
func (m *ObjectPool) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ObjectPool.Marshal(b, m, deterministic)
}
func (m *ObjectPool) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ObjectPool.Merge(m, src)
}
func (m *ObjectPool) XXX_Size() int {
	return xxx_messageInfo_ObjectPool.Size(m)
}
func (m *ObjectPool) XXX_DiscardUnknown() {
	xxx_messageInfo_ObjectPool.DiscardUnknown(m)
}

var xxx_messageInfo_ObjectPool proto.InternalMessageInfo

func (m *ObjectPool) GetRepository() *Repository {
	if m != nil {
		return m.Repository
	}
	return nil
}

type PaginationParameter struct {
	// Instructs pagination to start sending results after the provided page
	// token appears. A page token allows for a generic pattern to uniquely
	// identify a result or 'page'. Each paginated RPC may interpret a page
	// token differently.
	PageToken string `protobuf:"bytes,1,opt,name=page_token,json=pageToken,proto3" json:"page_token,omitempty"`
	// When fully consuming the response the client will receive _at most_
	// `limit` number of resulting objects. Note that the number of response
	// messages might be much lower, as some response messages already send
	// multiple objects per message.
	// When the limit is smaller than 0, it will be normalized to 2147483647
	// on the server side. When limit is not set, it defaults to 0, and no
	// results are send in the response.
	Limit                int32    `protobuf:"varint,2,opt,name=limit,proto3" json:"limit,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *PaginationParameter) Reset()         { *m = PaginationParameter{} }
func (m *PaginationParameter) String() string { return proto.CompactTextString(m) }
func (*PaginationParameter) ProtoMessage()    {}
func (*PaginationParameter) Descriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{9}
}

func (m *PaginationParameter) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_PaginationParameter.Unmarshal(m, b)
}
func (m *PaginationParameter) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_PaginationParameter.Marshal(b, m, deterministic)
}
func (m *PaginationParameter) XXX_Merge(src proto.Message) {
	xxx_messageInfo_PaginationParameter.Merge(m, src)
}
func (m *PaginationParameter) XXX_Size() int {
	return xxx_messageInfo_PaginationParameter.Size(m)
}
func (m *PaginationParameter) XXX_DiscardUnknown() {
	xxx_messageInfo_PaginationParameter.DiscardUnknown(m)
}

var xxx_messageInfo_PaginationParameter proto.InternalMessageInfo

func (m *PaginationParameter) GetPageToken() string {
	if m != nil {
		return m.PageToken
	}
	return ""
}

func (m *PaginationParameter) GetLimit() int32 {
	if m != nil {
		return m.Limit
	}
	return 0
}

// https://git-scm.com/docs/git/#_options
type GlobalOptions struct {
	// Treat pathspecs literally (i.e. no globbing, no pathspec magic)
	LiteralPathspecs     bool     `protobuf:"varint,1,opt,name=literal_pathspecs,json=literalPathspecs,proto3" json:"literal_pathspecs,omitempty"`
	XXX_NoUnkeyedLiteral struct{} `json:"-"`
	XXX_unrecognized     []byte   `json:"-"`
	XXX_sizecache        int32    `json:"-"`
}

func (m *GlobalOptions) Reset()         { *m = GlobalOptions{} }
func (m *GlobalOptions) String() string { return proto.CompactTextString(m) }
func (*GlobalOptions) ProtoMessage()    {}
func (*GlobalOptions) Descriptor() ([]byte, []int) {
	return fileDescriptor_d8a4e87e678c5ced, []int{10}
}

func (m *GlobalOptions) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_GlobalOptions.Unmarshal(m, b)
}
func (m *GlobalOptions) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_GlobalOptions.Marshal(b, m, deterministic)
}
func (m *GlobalOptions) XXX_Merge(src proto.Message) {
	xxx_messageInfo_GlobalOptions.Merge(m, src)
}
func (m *GlobalOptions) XXX_Size() int {
	return xxx_messageInfo_GlobalOptions.Size(m)
}
func (m *GlobalOptions) XXX_DiscardUnknown() {
	xxx_messageInfo_GlobalOptions.DiscardUnknown(m)
}

var xxx_messageInfo_GlobalOptions proto.InternalMessageInfo

func (m *GlobalOptions) GetLiteralPathspecs() bool {
	if m != nil {
		return m.LiteralPathspecs
	}
	return false
}

func init() {
	proto.RegisterEnum("gitaly.ObjectType", ObjectType_name, ObjectType_value)
	proto.RegisterEnum("gitaly.SignatureType", SignatureType_name, SignatureType_value)
	proto.RegisterType((*Repository)(nil), "gitaly.Repository")
	proto.RegisterType((*CommitTrailer)(nil), "gitaly.CommitTrailer")
	proto.RegisterType((*GitCommit)(nil), "gitaly.GitCommit")
	proto.RegisterType((*CommitAuthor)(nil), "gitaly.CommitAuthor")
	proto.RegisterType((*ExitStatus)(nil), "gitaly.ExitStatus")
	proto.RegisterType((*Branch)(nil), "gitaly.Branch")
	proto.RegisterType((*Tag)(nil), "gitaly.Tag")
	proto.RegisterType((*User)(nil), "gitaly.User")
	proto.RegisterType((*ObjectPool)(nil), "gitaly.ObjectPool")
	proto.RegisterType((*PaginationParameter)(nil), "gitaly.PaginationParameter")
	proto.RegisterType((*GlobalOptions)(nil), "gitaly.GlobalOptions")
}

func init() { proto.RegisterFile("shared.proto", fileDescriptor_d8a4e87e678c5ced) }

var fileDescriptor_d8a4e87e678c5ced = []byte{
	// 940 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0x8c, 0x55, 0x5d, 0x6f, 0xdb, 0x36,
	0x17, 0xae, 0x65, 0xf9, 0xeb, 0xd8, 0xce, 0xab, 0xb2, 0x79, 0x31, 0x21, 0x43, 0x51, 0x4f, 0x03,
	0x86, 0xa0, 0xcb, 0xec, 0xd6, 0xeb, 0xbe, 0x80, 0xde, 0xc4, 0x5d, 0x16, 0x24, 0x5b, 0x6d, 0x43,
	0x71, 0xb0, 0x61, 0x37, 0x02, 0x6d, 0xb1, 0x0c, 0x57, 0x4a, 0x14, 0x48, 0x3a, 0x98, 0x73, 0xbd,
	0xab, 0x5d, 0xed, 0x97, 0xec, 0x67, 0xec, 0xff, 0xec, 0x1f, 0x0c, 0x24, 0x25, 0xc7, 0xd9, 0xb2,
	0xa1, 0x77, 0x3c, 0x0f, 0x1f, 0x1e, 0x9d, 0x8f, 0xe7, 0x1c, 0x41, 0x4f, 0x5d, 0x61, 0x49, 0xd2,
	0x61, 0x21, 0x85, 0x16, 0xa8, 0x49, 0x99, 0xc6, 0x7c, 0x73, 0xf0, 0x84, 0x0a, 0x41, 0x39, 0x19,
	0x59, 0x74, 0xb9, 0x7e, 0x33, 0xd2, 0x2c, 0x23, 0x4a, 0xe3, 0xac, 0x70, 0xc4, 0x03, 0xe0, 0x2c,
	0xd7, 0xee, 0x1c, 0xfd, 0xee, 0x01, 0xc4, 0xa4, 0x10, 0x8a, 0x69, 0x21, 0x37, 0xe8, 0x03, 0xe8,
	0x29, 0x2d, 0x24, 0xa6, 0x24, 0xc9, 0x71, 0x46, 0x42, 0x6f, 0x50, 0x3b, 0xec, 0xc4, 0xdd, 0x12,
	0x9b, 0xe2, 0x8c, 0xa0, 0x0f, 0xa1, 0x2f, 0x09, 0xc7, 0x9a, 0x5d, 0x93, 0xa4, 0xc0, 0xfa, 0x2a,
	0xac, 0x5b, 0x4e, 0xaf, 0x02, 0xe7, 0x58, 0x5f, 0xa1, 0x67, 0xb0, 0x4f, 0x99, 0x4e, 0xc4, 0xf2,
	0x27, 0xb2, 0xd2, 0x49, 0xca, 0x24, 0x59, 0x19, 0xff, 0xa1, 0x6f, 0xb9, 0x88, 0x32, 0x3d, 0xb3,
	0x57, 0x5f, 0x57, 0x37, 0xe8, 0x14, 0x06, 0xe6, 0x05, 0xe6, 0x9a, 0xc8, 0x1c, 0x6b, 0xf2, 0xf7,
	0xb7, 0x8c, 0xa8, 0xb0, 0x31, 0xa8, 0x1f, 0x76, 0xe2, 0xc7, 0x94, 0xe9, 0xe3, 0x8a, 0x76, 0xd7,
	0x0d, 0x23, 0xca, 0xc4, 0x47, 0x79, 0x22, 0xb7, 0x39, 0x85, 0x4d, 0x17, 0x1f, 0xe5, 0x3b, 0x79,
	0x7e, 0x04, 0xff, 0xa3, 0x3c, 0x29, 0xa4, 0xb0, 0xdf, 0xb0, 0x69, 0xb4, 0x2d, 0xad, 0x4f, 0xf9,
	0xdc, 0xa1, 0x26, 0x8f, 0x73, 0xbf, 0x5d, 0x0b, 0xbc, 0x73, 0xbf, 0xdd, 0x0a, 0xda, 0xb1, 0x6f,
	0x68, 0xd1, 0x17, 0xd0, 0x7f, 0x25, 0xb2, 0x8c, 0xe9, 0x85, 0xc4, 0x8c, 0x13, 0x89, 0x02, 0xa8,
	0xbf, 0x25, 0x9b, 0xb0, 0x36, 0xa8, 0x1d, 0xf6, 0x62, 0x73, 0x44, 0xfb, 0xd0, 0xb8, 0xc6, 0x7c,
	0xed, 0xaa, 0xd7, 0x8b, 0x9d, 0x11, 0xfd, 0xe9, 0x41, 0xe7, 0x94, 0x69, 0xf7, 0x18, 0xed, 0x81,
	0xc7, 0x52, 0xfb, 0xa8, 0x13, 0x7b, 0x2c, 0x45, 0x21, 0xb4, 0xd4, 0xda, 0xe6, 0x52, 0xbe, 0xaa,
	0x4c, 0x84, 0xc0, 0x5f, 0x8a, 0x74, 0x63, 0xcb, 0xdc, 0x8b, 0xed, 0x19, 0x1d, 0x41, 0x13, 0xaf,
	0xf5, 0x95, 0x90, 0xb6, 0xa0, 0xdd, 0xf1, 0xfe, 0xd0, 0xf5, 0x7e, 0xe8, 0xbc, 0x1f, 0xdb, 0xbb,
	0xb8, 0xe4, 0xa0, 0x31, 0x74, 0x56, 0x16, 0xd7, 0x44, 0x86, 0x8d, 0xff, 0x78, 0x70, 0x4b, 0x43,
	0x8f, 0x01, 0x0a, 0x2c, 0x49, 0xae, 0x13, 0x96, 0xaa, 0xb0, 0x69, 0x0b, 0xdf, 0x71, 0xc8, 0x59,
	0xaa, 0xd0, 0xfb, 0xd0, 0x31, 0x81, 0x24, 0x8a, 0xdd, 0x90, 0xb0, 0x35, 0xa8, 0x1d, 0xd6, 0xe3,
	0xb6, 0x01, 0x2e, 0xd8, 0x0d, 0x41, 0x2f, 0x61, 0x4f, 0x31, 0x9a, 0x63, 0xbd, 0x96, 0x24, 0xd1,
	0x9b, 0x82, 0xd8, 0xda, 0xee, 0x8d, 0xff, 0x5f, 0x7d, 0xf4, 0xa2, 0xba, 0x5d, 0x6c, 0x0a, 0x12,
	0xf7, 0xd5, 0xae, 0x89, 0xde, 0x83, 0x96, 0x96, 0x84, 0x24, 0x2c, 0x0d, 0x3b, 0xb6, 0x3c, 0x4d,
	0x63, 0x9e, 0xa5, 0xe8, 0x39, 0xb4, 0xb5, 0xab, 0xb9, 0x0a, 0x61, 0x50, 0x3f, 0xec, 0xde, 0x3a,
	0xbc, 0xd3, 0x91, 0x78, 0x4b, 0x8b, 0x7e, 0xa9, 0x41, 0x6f, 0x37, 0x43, 0x53, 0x4c, 0xab, 0x6b,
	0xd7, 0x2d, 0x7b, 0x36, 0xed, 0x22, 0x19, 0x66, 0xbc, 0x6a, 0x97, 0x35, 0xd0, 0x10, 0xfc, 0x14,
	0x6b, 0x62, 0xcb, 0xde, 0x1d, 0x1f, 0x0c, 0xdd, 0x50, 0x0d, 0xab, 0xa1, 0x1a, 0x2e, 0xaa, 0xa1,
	0x8a, 0x2d, 0x0f, 0x1d, 0x40, 0xdb, 0xcc, 0xd9, 0x8d, 0xc8, 0x89, 0x6d, 0x4a, 0x2f, 0xde, 0xda,
	0x51, 0x04, 0x70, 0xf2, 0x33, 0xd3, 0x17, 0x1a, 0xeb, 0xb5, 0xba, 0x95, 0x87, 0x09, 0xa2, 0x51,
	0xc9, 0x63, 0x01, 0xcd, 0x89, 0xc4, 0xf9, 0xea, 0xea, 0xde, 0x18, 0x3f, 0x87, 0xbe, 0xc6, 0x92,
	0x12, 0x9d, 0xb8, 0x16, 0xd9, 0x58, 0xbb, 0xe3, 0x87, 0x55, 0x01, 0xb6, 0xc2, 0x8a, 0x7b, 0x8e,
	0xe7, 0xac, 0xe8, 0x57, 0x0f, 0xea, 0x0b, 0x4c, 0xef, 0xf5, 0xe9, 0x24, 0xe8, 0x6d, 0x25, 0xf8,
	0x8f, 0x6f, 0xd4, 0xdf, 0xe9, 0x1b, 0x46, 0xba, 0x19, 0x51, 0x0a, 0xd3, 0x2a, 0xf1, 0xca, 0x34,
	0xdb, 0xa4, 0x3c, 0x3a, 0xa1, 0x34, 0xac, 0x50, 0xba, 0x25, 0x66, 0xb5, 0x72, 0x04, 0x4d, 0x8d,
	0x29, 0x25, 0xd2, 0x8e, 0xe9, 0xbf, 0x2a, 0xd9, 0x71, 0xee, 0x51, 0x56, 0xeb, 0xdd, 0x95, 0x15,
	0xbd, 0x01, 0xff, 0x52, 0x11, 0x89, 0x1e, 0x41, 0x83, 0xf2, 0x64, 0x3b, 0x7e, 0x3e, 0xe5, 0x67,
	0xe9, 0xb6, 0x42, 0xde, 0x7d, 0xca, 0xa8, 0xef, 0x2a, 0xe3, 0x09, 0x74, 0x29, 0x4f, 0xd6, 0xca,
	0x2c, 0xa0, 0x8c, 0x94, 0x2b, 0x0d, 0x28, 0xbf, 0x2c, 0x91, 0xe8, 0x1b, 0x00, 0xb7, 0x96, 0xe6,
	0x42, 0x70, 0xf4, 0x25, 0xc0, 0xce, 0x32, 0xaa, 0xd9, 0x2c, 0x51, 0x15, 0xef, 0xed, 0x4a, 0x9a,
	0xf8, 0xbf, 0xfd, 0x71, 0x54, 0x8b, 0x77, 0xb8, 0xd1, 0x39, 0x3c, 0x9a, 0x63, 0xca, 0x72, 0xac,
	0x99, 0xc8, 0xe7, 0x58, 0xe2, 0x8c, 0x6c, 0x47, 0x93, 0x92, 0x44, 0x8b, 0xb7, 0x24, 0x2f, 0x73,
	0xe8, 0x18, 0x64, 0x61, 0x00, 0x13, 0x34, 0x67, 0x95, 0x44, 0x1a, 0xb1, 0x33, 0xa2, 0x97, 0xd0,
	0x3f, 0xe5, 0x62, 0x89, 0xf9, 0xac, 0x30, 0xde, 0x14, 0xfa, 0x18, 0x1e, 0x72, 0xa6, 0x89, 0xc4,
	0xdc, 0xae, 0x3f, 0x55, 0x90, 0x95, 0xb2, 0xce, 0xda, 0x71, 0x50, 0x5e, 0xcc, 0x2b, 0xfc, 0xe9,
	0xa4, 0xca, 0xc8, 0x4e, 0x68, 0x17, 0x5a, 0x97, 0xd3, 0x6f, 0xa7, 0xb3, 0xef, 0xa7, 0xc1, 0x03,
	0x04, 0xd0, 0x7c, 0x35, 0x7b, 0xfd, 0xfa, 0x6c, 0x11, 0xd4, 0x50, 0x1b, 0xfc, 0xc9, 0x77, 0xb3,
	0x49, 0xe0, 0x99, 0xd3, 0x22, 0x3e, 0x39, 0x09, 0xea, 0xa8, 0x05, 0xf5, 0xc5, 0xf1, 0x69, 0xe0,
	0x3f, 0x3d, 0x82, 0xfe, 0x9d, 0xee, 0x18, 0xce, 0x74, 0x36, 0x3d, 0x09, 0x1e, 0x18, 0xce, 0xfc,
	0x74, 0xee, 0x1c, 0xfc, 0xf0, 0xd9, 0xb3, 0xaf, 0x02, 0x6f, 0xf2, 0xe2, 0xc7, 0x31, 0x65, 0x9a,
	0xe3, 0xe5, 0x70, 0x25, 0xb2, 0x91, 0x3b, 0x7e, 0x22, 0x24, 0x1d, 0xb9, 0xc2, 0x8d, 0xae, 0x9f,
	0xbf, 0x70, 0x3f, 0xb8, 0x11, 0x15, 0x25, 0x56, 0x2c, 0x97, 0x4d, 0x0b, 0x7d, 0xfa, 0x57, 0x00,
	0x00, 0x00, 0xff, 0xff, 0xc2, 0x95, 0x47, 0xc7, 0x19, 0x07, 0x00, 0x00,
}
